package com.cn.tianji.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.cn.tianji.entity.TorqueWarnLog;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 扭矩预警日志表;(torque_warn_log)表服务接口
 * @author : http://www.chiner.pro
 * @date : 2024-6-13
 */
public interface TorqueWarnLogService extends IService<TorqueWarnLog> {
    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    TorqueWarnLog queryById(String id);
    /**
     * 新增数据
     *
     * @param torqueWarnLog 实例对象
     * @return 实例对象
     */
    TorqueWarnLog insert(TorqueWarnLog torqueWarnLog);
    /**
     * 更新数据
     *
     * @param torqueWarnLog 实例对象
     * @return 实例对象
     */
    TorqueWarnLog update(TorqueWarnLog torqueWarnLog);

    /**
     * 生成扭矩预警日志数据实体类
     *
     * @param jh 井号
     * @param workingCondition 预警点工况
     * @param motion 预警点动作
     * @param warnStartTime 预警取值开始时间
     * @param warnEndTime 预警取值结束时间
     * @param warnTorque 预警点扭矩
     * @param warnMeanValue 预警点扭矩均值
     * @param warnFluctuatedValue 预警点扭矩波动值
     * @param warnBitdepth 预警点钻头位置
     * @param warnDepth 预警点井深
     * @param referenceStartTime 参考值起点时间
     * @param referenceEndTime 参考值结束时间
     * @param referenceMeanValue 参考值扭矩均值
     * @param referenceFluctuatedValue 参考值扭矩波动值
     * @param resMeanValue 均值计算结果值
     * @param resFluctuatedValue 波动值计算结果值
     * @param thresholdMeanValue 均值阈值
     * @param thresholdFluctuatedValue 波动值阈值
     * @return
     */
    TorqueWarnLog createTorqueWarnLog(String jh, String workingCondition , String motion, Date warnStartTime, Date warnEndTime, BigDecimal warnTorque, BigDecimal warnMeanValue, BigDecimal warnFluctuatedValue,
                                      BigDecimal warnBitdepth, BigDecimal warnDepth, Date referenceStartTime, Date referenceEndTime, BigDecimal referenceMeanValue, BigDecimal referenceFluctuatedValue, BigDecimal resMeanValue,
                                      BigDecimal resFluctuatedValue, BigDecimal thresholdMeanValue, BigDecimal thresholdFluctuatedValue,String isMeanWarn,String isFluctuatedWarn
    );

    /**
     *  新增扭矩预警日志
     * @param model 扭矩实体类数据
     */
    void saveTorqueWarnLog(TorqueWarnLog model);
}
