package com.cn.tianji.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.conditions.update.LambdaUpdateChainWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cn.tianji.entity.HookLoadWarnLog;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.mapper.HookLoadWarnLogMapper;
import com.cn.tianji.service.HookLoadWarnLogService;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;

@Service
public class HookLoadWarnLogServiceImpl  extends ServiceImpl<HookLoadWarnLogMapper, HookLoadWarnLog> implements HookLoadWarnLogService {

    @Autowired
    private HookLoadWarnLogMapper hookLoadWarnLogMapper;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public HookLoadWarnLog queryById(String id){
        return hookLoadWarnLogMapper.selectById(id);
    }

    /**
     * 新增数据
     *
     * @param hookLoadWarnLog 实例对象
     * @return 实例对象
     */
    @Override
    public HookLoadWarnLog insert(HookLoadWarnLog hookLoadWarnLog){
        hookLoadWarnLogMapper.insert(hookLoadWarnLog);
        return hookLoadWarnLog;
    }

    /**
     * 更新数据
     *
     * @param hookLoadWarnLog 实例对象
     * @return 实例对象
     */
    @Override
    public HookLoadWarnLog update(HookLoadWarnLog hookLoadWarnLog){
        //1. 根据条件动态更新
        LambdaUpdateChainWrapper<HookLoadWarnLog> chainWrapper = new LambdaUpdateChainWrapper<HookLoadWarnLog>(hookLoadWarnLogMapper);
        if(StrUtil.isNotBlank(hookLoadWarnLog.getJh())){
            chainWrapper.eq(HookLoadWarnLog::getJh, hookLoadWarnLog.getJh());
        }
        if(StrUtil.isNotBlank(hookLoadWarnLog.getMotion())){
            chainWrapper.eq(HookLoadWarnLog::getMotion, hookLoadWarnLog.getMotion());
        }
        if(StrUtil.isNotBlank(hookLoadWarnLog.getWorkingCondition())){
            chainWrapper.eq(HookLoadWarnLog::getWorkingCondition, hookLoadWarnLog.getWorkingCondition());
        }
        if(StrUtil.isNotBlank(hookLoadWarnLog.getWarnDes())){
            chainWrapper.eq(HookLoadWarnLog::getWarnDes, hookLoadWarnLog.getWarnDes());
        }
        if(StrUtil.isNotBlank(hookLoadWarnLog.getRemake())){
            chainWrapper.eq(HookLoadWarnLog::getRemake, hookLoadWarnLog.getRemake());
        }
        //2. 设置主键，并更新
        chainWrapper.set(HookLoadWarnLog::getId, hookLoadWarnLog.getId());
        boolean ret = chainWrapper.update();
        //3. 更新成功了，查询最最对象返回
        if(ret){
            return queryById(hookLoadWarnLog.getId());
        }else{
            return hookLoadWarnLog;
        }
    }

    /**
     * 插入预警数据
     * @param nowPoint 当前点数据
     * @param workingCondition 工况
     * @param workingConditionType 预警点工况类别:起钻、下钻、起钻——活动钻具、下钻-活动钻具、钻进活动钻具
     * @param motion 动作
     * @param referenceStartTime 参考柱起点时间
     * @param referenceEndTime 参考柱结束时间
     * @param referenceMaxWoh 参考柱钩载最大值
     * @param referenceMinWoh 参考柱钩载最小值
     * @param referenceWoh 参考柱钩载值
     * @param columnWeight 钻杆重量
     * @param warnValue 预警值
     */
    @SneakyThrows
    @Override
    public void saveHookLoadWarnLog(WellDrilling nowPoint, String workingCondition, String workingConditionType, String motion,
                                    String referenceStartTime, String referenceEndTime, BigDecimal referenceWoh, BigDecimal referenceMaxWoh, BigDecimal referenceMinWoh,
                                    BigDecimal columnWeight, BigDecimal warnValue){
        String warnDesc="符合钩载变化异常预警条件:"+
                " <br> 当前点时间" + nowPoint.getTime()+"; 当前点大钩负荷:" + nowPoint.getWoh() + " KN,"  +
                ",<br> 参考柱起点时间:" + referenceStartTime + ",参考柱结束时间:" + referenceEndTime +",参考柱钩载:" + referenceWoh + " KN, "+
                ",钻杆重量:" + columnWeight + " KN,<br> 大于预警值:" + warnValue + " KN ";
        HookLoadWarnLog model=new HookLoadWarnLog();
        model.setWarnTime(sdf.parse(nowPoint.getTime()));
        model.setWarnWoh(nowPoint.getWoh());
        model.setWarnHkh(nowPoint.getHkh());
        model.setWarnBitdepth(nowPoint.getBitdepth());
        model.setWorkingCondition(workingCondition);
        model.setWorkingConditionType(workingConditionType);
        model.setJh(nowPoint.getJh());
        model.setMotion(motion);
        model.setReferenceStartTime(sdf.parse(referenceStartTime));
        model.setReferenceEndTime(sdf.parse(referenceEndTime));
        model.setReferenceWoh(referenceWoh);
        model.setReferenceMaxWoh(referenceMaxWoh);
        model.setReferenceMinWoh(referenceMinWoh);
        model.setColumnWeight(columnWeight);
        model.setWarnValue(warnValue);
        model.setWarnDes(warnDesc);
        model.setCreateTime(new Date());
        hookLoadWarnLogMapper.insert(model);
    }

    /**
     * 插入活动钻具钩载预警数据
     * @param jh 井号
     * @param workingCondition 工况
     * @param workingConditionType 预警点工况类别:起钻、下钻、起钻——活动钻具、下钻-活动钻具、钻进活动钻具
     * @param motion 动作
     * @param mobilDrillingToolsStartTime 活动钻具起点时间
     * @param mobilDrillingToolsEndTime 活动钻具结束时间
     * @param mobilDrillingToolsWoh 活动钻具钩载
     * @param mobilDrillingToolsMaxWoh 活动钻具钩载最大值
     * @param mobilDrillingToolsMinWoh 活动组那句钩载最小值
     * @param referenceStartTime 参考柱起点时间
     * @param referenceEndTime 参考柱结束时间
     * @param referenceWoh 参考柱钩载值
     * @param referenceMaxWoh 参考柱钩载最大值
     * @param referenceMinWoh 参考柱钩载最小值
     * @param warnValue 预警值
     */
    @SneakyThrows
    @Override
    public void saveMobilDrillingToolsHookLoadWarnLog(String jh,String workingCondition,String workingConditionType,String motion,
                                                      String mobilDrillingToolsStartTime,String mobilDrillingToolsEndTime,BigDecimal mobilDrillingToolsWoh,BigDecimal mobilDrillingToolsMaxWoh,BigDecimal mobilDrillingToolsMinWoh,
                                                      String referenceStartTime,String referenceEndTime,BigDecimal referenceWoh, BigDecimal referenceMaxWoh,BigDecimal referenceMinWoh,
                                                      BigDecimal warnValue){
        String warnDesc="符合钩载变化异常预警条件 "+
                "<br> 活动钻具钩载波动值:"+ mobilDrillingToolsWoh +" KN, 活动钻具起点时间:"+mobilDrillingToolsStartTime+" ,活动钻具结束时间:"+mobilDrillingToolsEndTime+
                ",<br> 活动钻具参考钩载:" + referenceWoh + " KN,参考柱起点时间:" + referenceStartTime + ",参考柱结束时间:" + referenceEndTime +
                ",<br> 大于预警值:" + warnValue + " KN ";
        HookLoadWarnLog model=new HookLoadWarnLog();
        model.setMobilDrillingToolsStartTime(sdf.parse(mobilDrillingToolsStartTime));
        model.setMobilDrillingToolsEndTime(sdf.parse(mobilDrillingToolsEndTime));
        model.setMobilDrillingToolsWoh(mobilDrillingToolsWoh);
        model.setMobilDrillingToolsMaxWoh(mobilDrillingToolsMaxWoh);
        model.setMobilDrillingToolsMinWoh(mobilDrillingToolsMinWoh);

        model.setJh(jh);
        model.setWarnTime(sdf.parse(mobilDrillingToolsEndTime));
        model.setMotion(motion);
        model.setWorkingCondition(workingCondition);
        model.setWorkingConditionType(workingConditionType);
        model.setReferenceStartTime(sdf.parse(referenceStartTime));
        model.setReferenceStartTime(sdf.parse(referenceEndTime));
        model.setReferenceMaxWoh(referenceMaxWoh);
        model.setReferenceMinWoh(referenceMinWoh);
        model.setReferenceWoh(referenceWoh);
        model.setWarnValue(warnValue);
        model.setWarnDes(warnDesc);
        model.setCreateTime(new Date());
        hookLoadWarnLogMapper.insert(model);
    }

    /**
     * 生成新的活动钻具算法预警实体类
     * @param jh 井号
     * @param workingCondition 工况
     * @param workingConditionType 预警点工况类别:起钻、下钻、起钻——活动钻具、下钻-活动钻具、钻进活动钻具
     * @param motion 动作
     * @param mobilDrillingToolsWoh 活动钻具钩载波动值
     * @param hkhSub 钻头位置差值
     * @param mobilDrillingToolsStartTime 活动钻具开始时间
     * @param mobilDrillingToolsMaxWoh 活动钻具钩载最大值
     * @param mobilDrillingToolsMinWoh 活动钻具钩载最小值
     * @param mobilDrillingToolsEndTime 活动钻具结束时间
     * @param hkhStart 钻头起点位置
     * @param hkhEnd 钻头结束位置
     * @param wohHkhRes 钩载除以钻头位置值
     * @param warnValue 预警值
     * @return HookLoadWarnLog
     */
    @Override
    @SneakyThrows
    public HookLoadWarnLog creatNewMobilDrillingToolsHookLoadWarnLog(String jh,String workingCondition,String workingConditionType,String motion,
                                                         BigDecimal mobilDrillingToolsWoh,BigDecimal hkhSub,String mobilDrillingToolsStartTime,
                                                         BigDecimal mobilDrillingToolsMaxWoh,BigDecimal mobilDrillingToolsMinWoh,
                                                         String mobilDrillingToolsEndTime,BigDecimal hkhStart,BigDecimal hkhEnd,
                                                         BigDecimal wohHkhRes,BigDecimal warnValue){
        HookLoadWarnLog model=new HookLoadWarnLog();
        String warnDesc="符合钩载变化异常预警条件 (活动钻具):"+
                "<br> 钩载波动值:"+mobilDrillingToolsWoh+" KN ;  钻头位置差值: "+hkhSub+" m ;"+
                "<br> 活动钻具起点时间:"+mobilDrillingToolsStartTime+"; 活动钻具结束时间"+mobilDrillingToolsEndTime+
                "<br> 大钩起点位置:"+hkhStart+" m ; 活动钻具结束位置"+hkhEnd+" m;"+
                "<br> 钩载波动值/钻头位置差值:"+wohHkhRes+" KN/m ; 预警值"+warnValue+" KN/m ;";
        model.setJh(jh);
        model.setMotion(motion);
        model.setWorkingCondition(workingCondition);
        model.setWorkingConditionType(workingConditionType);

        if(!StrUtil.isEmpty(mobilDrillingToolsEndTime)){
            model.setWarnTime(sdf.parse(mobilDrillingToolsEndTime));
        }
        model.setWarnWoh(mobilDrillingToolsWoh);
        model.setWarnBitdepth(hkhEnd);

        if(!StrUtil.isEmpty(mobilDrillingToolsStartTime)) {
            model.setMobilDrillingToolsStartTime(sdf.parse(mobilDrillingToolsStartTime));
        }
        if(!StrUtil.isEmpty(mobilDrillingToolsEndTime)) {
            model.setMobilDrillingToolsEndTime(sdf.parse(mobilDrillingToolsEndTime));
        }
        model.setMobilDrillingToolsWoh(mobilDrillingToolsWoh);
        model.setMobilDrillingToolsMaxWoh(mobilDrillingToolsMaxWoh);
        model.setMobilDrillingToolsMinWoh(mobilDrillingToolsMinWoh);

        model.setHkhStart(hkhStart);
        model.setHkhEnd(hkhEnd);
        model.setHkhSub(hkhSub);
        model.setHkhWoh(wohHkhRes);

        model.setWarnValue(warnValue);
        model.setWarnDes(warnDesc);
        model.setCreateTime(new Date());
        return model;
    }
}
