package com.cn.tianji.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cn.tianji.common.CommonUtils;
import com.cn.tianji.common.HttpUtil;
import com.cn.tianji.entity.SyncRecord;
import com.cn.tianji.entity.WellDrillingSICP;
import com.cn.tianji.entity.WellName;
import com.cn.tianji.service.PullDataSICPService;
import com.cn.tianji.service.SyncRecordService;
import com.cn.tianji.service.WellDrillingSICPService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

/**
 * SICP数据采集业务逻辑层
 *
 * @author Zhao RenJi
 */
@Service
@Slf4j
public class PullDataSICPServiceImpl implements PullDataSICPService {

    @Resource
    private WellDrillingSICPService wellDrillingSICPService;

    @Resource
    private SyncRecordService syncRecordService;

    SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Autowired
    private ApplicationContext context;


    @Override
    public void createTables(List<String> jhs){
        jhs.forEach(item->{
            String jhTableName = "LTD_" + item.replace('-', '_').toUpperCase().trim();
            wellDrillingSICPService.createTable(jhTableName,System.currentTimeMillis());
        });
    }

    /**
     * 更新采集井数据列表
     */
    @Override
    public void updatePullJhList(){
        //获取最新在线井数据
        List<WellName> allJhSicp = getAllJhSicp();
        //获取数据库中所有井表名
        List<String> tablesFromDbAll = wellDrillingSICPService.getTablesFromDb();
        //log.info("所有的数据表:{}",tablesFromDbAll);
        //根据前缀筛选表名数据
        List<String> tablesFromDb =filterEntitiesByPrefix(tablesFromDbAll,"SICP");
        //tablesFromDb.forEach(item->{item.replace("\"","");});
        log.info("数据库SICP_表名:{}",tablesFromDb);
        allJhSicp.forEach(item->{
            String sicpTableName=item.getWellTableNameSicp();
            //log.info("sicpTableName:{}",sicpTableName);
            if(!tablesFromDb.contains(("\""+sicpTableName+"\""))){
                //数据库中不存在该井表创建该井表数据
                log.info("创建井表:{}",sicpTableName);
                wellDrillingSICPService.createTable("\""+sicpTableName+"\"",System.currentTimeMillis());
                //插入该井采集数据;当前时间减去3分钟
                Date nowDate=new Date();
                Date specifiedTime = CommonUtils.getSpecifiedTime(nowDate, Calendar.MINUTE, -3);
                SyncRecord model =new SyncRecord();
                model.setId(UUID.randomUUID().toString());
                model.setCollectTime(sdf.format(specifiedTime));
                model.setTableType("SICP");
                model.setTableName(sicpTableName);
                log.info("新增采集sicp井数据：{}",model);
                syncRecordService.save(model);
            }
        });
        //批量采集数据
        //collectWelLogBatch(allJhSicp);
        listGrouping(allJhSicp);
    }


    /**
     * 对数据进行分组,每10条数据为一组
     * @param allList
     */
    public void listGrouping(List<WellName> allList){
         ExecutorService fifoThreadPool=(ThreadPoolExecutor)context.getBean("fifoThreadPool");
        List<List<WellName>> groupData = groupData(allList, 10);
        groupData.forEach(item->{
            fifoThreadPool.submit(() -> {
                log.info("执行任务executed by：{}"  , Thread.currentThread().getName());
                collectWelLogBatch(allList);
            });
        });
    }

    /**
     * 数据分组方法
     * @param dataList
     * @param groupSize
     * @return
     */
    public static List<List<WellName>> groupData(List<WellName> dataList, int groupSize) {
        List<List<WellName>> groupedData = new ArrayList<>();
        for (int i = 0; i < dataList.size(); i += groupSize) {
            int end = Math.min(i + groupSize, dataList.size());
            groupedData.add(dataList.subList(i, end));
        }
        return groupedData;
    }

    /**
     * 批量采集录井数据
     */
    public void collectWelLogBatch(List<WellName> allList){
        log.info("分组数据:{}",allList);
        Date startDate=new Date();
        log.info("开始时间:{}",sdf.format(startDate));
        //根据井表名批量查询本次所有采集井上次采集时间数据
//        allList.forEach(i->{i.setWellTableNameSicp("\""+i.getWellTableNameSicp()+"\"");});
        List<String> jhTables = allList.stream().map(WellName::getWellTableNameSicp).collect(Collectors.toList());
        List<SyncRecord> syncRecordList = getSyncRecordList(jhTables);
        //循环所有得井
        allList.forEach(item->{
            //根据井名查询数据采集表该井上次数据采集时间
            List<SyncRecord> syncRecords = syncRecordList.stream().filter(items -> items.getTableName().equals(item.getWellTableNameSicp())).collect(Collectors.toList());
            //判断该井是否存在数据采集时间
            SyncRecord syncRecord = new SyncRecord();
            String collectTime="";
            //该井表是否存在最新采集时间
            if(syncRecords.size()>0){
                syncRecord=syncRecords.get(0);
                collectTime = syncRecord.getCollectTime();
            }else{
                //不存在查询该井表最新采集时间
                String jhTableName=item.getWellTableNameSicp().replace("\"","");
                String maxTime = wellDrillingSICPService.selectMaxTime(jhTableName);
                if(maxTime != null){
                    collectTime=maxTime;
                }else{
                    Date nowDate=new Date();
                    Date specifiedTime = CommonUtils.getSpecifiedTime(nowDate, Calendar.MINUTE, -3);
                    collectTime= sdf.format(specifiedTime);
                }
                syncRecord.setId(UUID.randomUUID().toString());
                syncRecord.setTableType("SICP");
                syncRecord.setTableName(item.getWellTableNameSicp());
                syncRecord.setCollectTime(collectTime);
                log.info("新增采集sicp井数据：{}",syncRecord);
                syncRecordService.save(syncRecord);
            }
            //根据井号获取该井录井数据
            List<WellDrillingSICP> sicpWellDataByJhTime = getSicpWellDataByJhTime(item.getWellName(), collectTime);
            log.info("采集井:井号:{},时间:{}",item.getWellTableNameSicp(),collectTime);
            if(sicpWellDataByJhTime.size()>0){
                //批量插入录井数据
                int i = wellDrillingSICPService.insertTableBatch(item.getWellTableNameSicp(), sicpWellDataByJhTime);
                //更新采集时间
                if(sicpWellDataByJhTime.size()>0){
                    WellDrillingSICP wellDrillingSICP = sicpWellDataByJhTime.get(sicpWellDataByJhTime.size() - 1);
                    String time = wellDrillingSICP.getTIME();
                    syncRecord.setCollectTime(time);
                }
            }
            //更新采集时间
            log.info("更新采集时间:{};井号:{}",syncRecord.getCollectTime(),syncRecord.getTableName());
            syncRecordService.updateById(syncRecord);

        });
        Date endDate=new Date();
        long l = CommonUtils.timeDifference(startDate, endDate);
        log.info("结束时间:{};耗时:{}秒",sdf.format(endDate),l);
    }


    public List<SyncRecord> getSyncRecordList(List<String> jhTables){
        QueryWrapper<SyncRecord> wrapper=new QueryWrapper<>();
        wrapper.lambda().in(SyncRecord::getTableName,jhTables).eq(SyncRecord::getTableType,"SICP");
        return syncRecordService.list(wrapper);
    }
    /**
     * 获取所有井数据
     *
     * @return
     */
    @Override
    public List<WellName> getAllJhSicp() {
        String url = "http://10.68.224.20:15221/v1/structuredData/serviceData/SICPALLJH?app_key=SICP";
        log.info("请求获取井号地址:{}", url);
        JSONObject params = new JSONObject();
        JSONObject data = postHttpData(url, params);
        List<WellName> wellNameList = new ArrayList<>();
        JSONArray rows = data.getJSONArray("rows");
        JSONObject o = JSONObject.parseObject(String.valueOf(rows.get(0)));
        String alljh = o.getString("alljh");
        String[] split = alljh.split("、");
        for (String jh : split) {
            WellName model = new WellName();
            model.setWellName(jh);
            model.setWellTableNameSicp(getJhTableByType(jh, "SICP"));
            model.setWellTableNameLtd(getJhTableByType(jh, "LTD"));
            wellNameList.add(model);
        }
        return wellNameList;
    }


    /**
     * 根据井号及时间采集单井录井数据
     *
     * @param jh
     * @param startTime
     */
    @Override
    public List<WellDrillingSICP> getSicpWellDataByJhTime(String jh, String startTime) {
        String url = "http://10.68.224.20:15221/v1/structuredData/serviceData/SICP3M?app_key=YF";
        JSONObject params = new JSONObject();
        JSONObject conditionJh = new JSONObject();
        conditionJh.put("condition", jh);
        JSONObject conditionTime = new JSONObject();
        conditionTime.put("condition", startTime);
        params.put("jh", conditionJh);
        params.put("kssj", conditionTime);
        JSONObject data = postHttpData(url, params);
        JSONArray rows = data.getJSONArray("rows");
        List<WellDrillingSICP> list = new ArrayList<>();
        for (Object item : rows) {
            // 打印传递的消息
            WellDrillingSICP model = JSONObject.parseObject(String.valueOf(item), WellDrillingSICP.class);
            String jrtime = model.getJRTIME();
            String time = model.getTIME();
            if (jrtime.contains("+")) {
                int i = jrtime.indexOf("+");
                String jrtimes = jrtime.substring(0, i);
                model.setJRTIME(jrtimes);
            }
            if (time.contains("+")) {
                int i = time.indexOf("+");
                String times = time.substring(0, i);
                model.setTIME(times);
            }
            list.add(model);
        }
        return list;
    }

    /**
     * post请求获取数据
     *
     * @param url
     * @param params
     * @return
     */
    public JSONObject postHttpData(String url, JSONObject params) {
        String resultJson = HttpUtil.sendPost(url, String.valueOf(params));
        JSONObject obj = JSONObject.parseObject(resultJson);
        String statusCode = obj.getString("statusCode");
        JSONObject res = new JSONObject();
        if ("000".equals(statusCode)) {
            res = obj.getJSONObject("data");
        }
        return res;
    }


    /**
     * 根据类别将类别加到井号前缀
     * @param jh 井号
     * @param type 类别
     * @return 井表名:LTD(SICP)_jh
     */
    public String getJhTableByType(String jh, String type) {
        return type + "_" + jh.replace("-", "_");
    }

    /**
     * 根据前缀筛选实体列表
     *
     * @param entities 原始实体列表
     * @param prefix   要筛选的前缀
     * @return 筛选后的实体列表
     */
    public List<String> filterEntitiesByPrefix(List<String> entities, String prefix) {
        log.info("筛选前的数据:{}",entities);
        log.info("筛选前的前缀:{}",prefix);
//        // 过滤出以prefix开头的字符串 收集结果到新的列表中
//        entities.stream().forEach(item->{
//            log.info(item.replace("\"",""));
//            log.info(String.valueOf(item.replace("\"","").startsWith("SICP")));
//        });
        List<String> filteredList = entities.stream().filter(item -> item.replace("\"","").startsWith(prefix)).collect(Collectors.toList());
        log.info("筛选后的数据:{}",filteredList);
        filteredList.forEach(item->item.replace("\"",""));
        return filteredList;
    }


}
