package com.cn.tianji.service.Impl;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cn.tianji.common.CommonUtils;
import com.cn.tianji.common.RedisUtil;
import com.cn.tianji.common.Result;
import com.cn.tianji.dto.ConditionDeterminationDto;
import com.cn.tianji.entity.*;
import com.cn.tianji.service.*;
import com.cn.tianji.util.CommonUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

@Service
@Slf4j
public class WorkingConditionCalculationServiceImpl implements WorkingConditionCalculationService {
    @Resource
    private WellDrillingService wellDrillingService;

    @Resource
    private ConditionIdentificationService conditionIdentificationService;

    @Resource
    private OperatingConditionThresholdService operatingConditionThresholdService;

    @Resource
    private WorkingConditionCalculationLogService workingConditionCalculationLogService;

    @Resource
    private DrillingHookloadService drillingHookloadService;

    @Resource
    private DrillingOutHookLoadService drillingOutHookLoadService;

    @Resource
    private HookLoadWarnLogService hookLoadWarnLogService;

    @Resource
    private TotalPoolVolumeService totalPoolVolumeService;

    @Resource
    private TotalPoolVolumeWarnLogService totalPoolVolumeWarnLogService;

    @Resource
    private PumpingPressureService pumpingPressureService;

    @Resource
    private PumpingPressureWarnLogService pumpingPressureWarnLogService;

    @Resource
    private TorqueService torqueService;

    @Resource
    private TorqueWarnLogService torqueWarnLogService;

    @Resource
    private YjWohWarnValueService wohWarnValueService;

    @Resource
    private YjTorqueWarnValueService torqueWarnValueService;

    @Resource
    private YjPumpingWarnValueService pumpingWarnValueService;


    @Resource
    private RedisUtil redisUtil;
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    /**
     * 根据井号,开始结束时间重新计算钻井工况动作
     *
     * @param workingConditionCalculationLogId
     */
    @Override
    public Result<?> workingConditionCalculation(String workingConditionCalculationLogId) {
        WorkingConditionCalculationLog workingConditionCalculationLogModel = workingConditionCalculationLogService.getById(workingConditionCalculationLogId);
        String jh = workingConditionCalculationLogModel.getJh();
        String startTime = workingConditionCalculationLogModel.getStartTime();
        String endTime = workingConditionCalculationLogModel.getEndTime();
        String calculateType = workingConditionCalculationLogModel.getCalculateType();
        String isDelRedis = workingConditionCalculationLogModel.getIsDelRedis();
        Result<Object> res = new Result<>();
        switch (calculateType) {
            case "0":
                //从开始时间往前取10秒的时间作为开始时间
                try {
                    Date startTimeDate = sdf.parse(startTime);
                    Date specifiedTime = CommonUtils.getSpecifiedTime(startTimeDate, Calendar.SECOND, -10);
                    startTime = sdf.format(specifiedTime);
                } catch (ParseException e) {
                    e.printStackTrace();
                }
                QueryWrapper<OperatingConditionThreshold> operatingConditionThresholdQueryWrapper = new QueryWrapper<>();
                operatingConditionThresholdQueryWrapper.lambda().eq(OperatingConditionThreshold::getJh, jh);
                OperatingConditionThreshold one = operatingConditionThresholdService.getOne(operatingConditionThresholdQueryWrapper);
                if (one != null) {
                    calWorkingCondition(jh, startTime, endTime, one, workingConditionCalculationLogModel);
                    res = Result.OK();
                } else {
                    res = Result.error("计算失败,无设置参数数据");
                }

                break;
            case "1":
                //钩载重新计算
                hookLoadWarningCalculation(jh, startTime, endTime, workingConditionCalculationLogModel, isDelRedis);
                res = Result.OK();
                break;
            case "2":
                //总池体积重新计算
                totalPoolVolumeWarningCalculation(jh, startTime, endTime, workingConditionCalculationLogModel, isDelRedis);
                res = Result.OK();
                break;
            case "3":
                //扭矩重新计算
                torqueWarningCalculation(jh, startTime, endTime, workingConditionCalculationLogModel, isDelRedis);
                res = Result.OK();
                break;
            case "4":
                //泵压重新计算
                pumpingPressureWarningCalculation(jh, startTime, endTime, workingConditionCalculationLogModel, isDelRedis);
                res = Result.OK();
                break;
            default:
                res = Result.error("计算失败,无计算类型");
                break;
        }
        return res;
    }

    /**
     * 计算工况动作并更新该点位数据
     *
     * @param jh
     * @param startTime
     * @param endTime
     * @param one
     */
//    @Async("workingConditionCalculationTaskExecutor")
    public void calWorkingCondition(String jh, String startTime, String endTime, OperatingConditionThreshold one, WorkingConditionCalculationLog workingConditionCalculationLogModel) {
        Thread currentThread = Thread.currentThread();
        String threadName = currentThread.getName();
        long threadId = currentThread.getId();
        Thread.State threadState = currentThread.getState();
        System.out.println("当前线程名称: " + threadName);
        System.out.println("当前线程ID: " + threadId);
        System.out.println("当前线程状态: " + threadState);
        //根据主键获取计算开始和结束时间
        //  WorkingConditionCalculationLog workingConditionCalculationLogModel = workingConditionCalculationLogService.getById(workingConditionCalculationLogId);
        //根据开始结束获取钻井数据
        List<WellDrilling> list = queryList(jh, startTime, endTime);
        String jhTableName = CommonUtils.getJhTableName(jh);
        for (int i = 2; i < list.size(); i++) {
            List<WellDrilling> calList = new ArrayList<>();
            WellDrilling nowPoint = list.get(i);
            WellDrilling perPoint = list.get(i - 2);
            calList.add(nowPoint);
            calList.add(perPoint);
            ConditionDeterminationDto conditionDeterminationDto = conditionIdentificationService.conditionDetermination(calList, one);
            nowPoint.setMotion(conditionDeterminationDto.getMotion());
            nowPoint.setOperatingMode(conditionDeterminationDto.getOperatingMode());
            log.info("更新点位时间:{};工况:{},动作:{}", nowPoint.getTime(), nowPoint.getMotion(), nowPoint.getOperatingMode());
            QueryWrapper<WellDrilling> wrapper1 = new QueryWrapper<>();
            wrapper1.apply("TO_DATE(SUBSTR(TIME, 1, 19), 'yyyy-MM-dd HH24:mi:ss') = to_date({0},'yyyy-mm-dd hh24:mi:ss')", nowPoint.getTime());
            wellDrillingService.updateByTimeOrcale(jhTableName, conditionDeterminationDto.getMotion(), conditionDeterminationDto.getOperatingMode(), wrapper1);
            if ("1".equals(workingConditionCalculationLogModel.getIsCalculate())) {
                workingConditionCalculationLogModel.setIsCalculate("2");
            }
            ;
            if (i == (list.size() - 1)) {
                workingConditionCalculationLogModel.setIsCalculate("3");
            }
            workingConditionCalculationLogModel.setCalTime(nowPoint.getTime());
            workingConditionCalculationLogService.updateById(workingConditionCalculationLogModel);
        }
    }

    /**
     * 钩载预警重新计算
     *
     * @param jh        井号
     * @param startTime 开始时间
     * @param endTime   结束时间
     */
    @SneakyThrows
    @Async("workingConditionCalculationTaskExecutor")
    public void hookLoadWarningCalculation(String jh, String startTime, String endTime, WorkingConditionCalculationLog workingConditionCalculationLogModel, String isDelRedis) {
        Thread currentThread = Thread.currentThread();
        String threadName = currentThread.getName();
        long threadId = currentThread.getId();
        Thread.State threadState = currentThread.getState();
        System.out.println("当前线程名称: " + threadName);
        System.out.println("当前线程ID: " + threadId);
        System.out.println("当前线程状态: " + threadState);
//        //根据主键获取计算开始和结束时间
//        WorkingConditionCalculationLog workingConditionCalculationLogModel = workingConditionCalculationLogService.getById(workingConditionCalculationLogId);
        BigDecimal warnValue = BigDecimal.valueOf(100);
        BigDecimal rodWeight = BigDecimal.valueOf(7.73);
        //根据开始结束获取钻井数据
        List<WellDrilling> list = queryList(jh, startTime, endTime);
        QueryWrapper<HookLoadWarnLog> hookLoadWarnLogQueryWrapper = new QueryWrapper<>();
        // hookLoadWarnLogQueryWrapper.lambda().between(HookLoadWarnLog::getWarnTime,sdf.parse(startTime),sdf.parse(endTime));
        // hookLoadWarnLogService.list(hookLoadWarnLogQueryWrapper);
        hookLoadWarnLogQueryWrapper.apply("WARN_TIME <= to_date({0},'yyyy-mm-dd hh24:mi:ss')", endTime);
        hookLoadWarnLogQueryWrapper.apply("WARN_TIME >= to_date({0},'yyyy-mm-dd hh24:mi:ss')", startTime);
        hookLoadWarnLogService.remove(hookLoadWarnLogQueryWrapper);
        redisUtil.del(jh + "_hook_load_recalculate");
        redisUtil.del(jh + "_drilling_hook_load_recalculate");
        for (int i = 0; i < list.size() - 1; i++) {
            WellDrilling nowPoint = list.get(i);
            String operatingMode = nowPoint.getOperatingMode();
            BigDecimal bitdepth = nowPoint.getBitdepth();
            int bitdepthThreshold = bitdepth.compareTo(BigDecimal.valueOf(500));
            if (bitdepthThreshold >= 0) {
                String redisHasKey = "";
                log.info("钩载预警计算,当前点井深{};时间:{}", nowPoint.getBitdepth(), nowPoint.getTime());
                if ("起下钻".equals(operatingMode)) {
                    redisHasKey = jh + "_hook_load_recalculate";
                    // drillingOutHookLoadService.hookLoadWarnCalculate(nowPoint, redisHasKey, warnValue, rodWeight);
                } else {
                    redisHasKey = jh + "_drilling_hook_load_recalculate";
                    drillingHookloadService.hookLoadWarnCalculate(nowPoint, redisHasKey, warnValue);
                }
            } else {
                log.info("钩载预警计算,钻头位置小于500米;当前点井深{};时间:{}", nowPoint.getBitdepth(), nowPoint.getTime());
            }
            if ("1".equals(workingConditionCalculationLogModel.getIsCalculate())) {
                workingConditionCalculationLogModel.setIsCalculate("2");
            }
            ;
            if (i == (list.size() - 2)) {
                workingConditionCalculationLogModel.setIsCalculate("3");
            }
            workingConditionCalculationLogModel.setCalTime(nowPoint.getTime());
            workingConditionCalculationLogService.updateById(workingConditionCalculationLogModel);

        }
    }

    /**
     * 总池体积预警重新计算
     *
     * @param jh        井号
     * @param startTime 开始时间
     * @param endTime   结束时间
     */
    @SneakyThrows
    @Async("workingConditionCalculationTaskExecutor")
    public void totalPoolVolumeWarningCalculation(String jh, String startTime, String endTime, WorkingConditionCalculationLog workingConditionCalculationLogModel, String isDelRedis) {
        Thread currentThread = Thread.currentThread();
        String threadName = currentThread.getName();
        long threadId = currentThread.getId();
        Thread.State threadState = currentThread.getState();
        System.out.println("当前线程名称: " + threadName);
        System.out.println("当前线程ID: " + threadId);
        System.out.println("当前线程状态: " + threadState);
//        BigDecimal warnValue = BigDecimal.valueOf(100);
//        BigDecimal rodWeight = BigDecimal.valueOf(7.73);
        //根据开始结束获取钻井数据
        List<WellDrilling> list = queryList(jh, startTime, endTime);
        QueryWrapper<TotalPoolVolumeWarnLog> totalPoolVolumeWarnLogQueryWrapper = new QueryWrapper<>();
        totalPoolVolumeWarnLogQueryWrapper.apply("WARN_TIME <= to_date({0},'yyyy-mm-dd hh24:mi:ss')", endTime);
        totalPoolVolumeWarnLogQueryWrapper.apply("WARN_TIME >= to_date({0},'yyyy-mm-dd hh24:mi:ss')", startTime);
        totalPoolVolumeWarnLogService.remove(totalPoolVolumeWarnLogQueryWrapper);
        String redisHasKey = jh + "_total_pool_volume_recalculate";
        if ("0".equals(isDelRedis)) {
            redisUtil.del(redisHasKey);
        }
        for (int i = 0; i < list.size() - 1; i++) {
            WellDrilling nowPoint = list.get(i);
            // String operatingMode = nowPoint.getOperatingMode();
            BigDecimal bitdepth = nowPoint.getBitdepth();
            int bitdepthThreshold = bitdepth.compareTo(BigDecimal.valueOf(500));
            if (bitdepthThreshold >= 0) {
//                try {
//                    totalPoolVolumeService.totalPoolVolumeWarn(nowPoint, redisHasKey);
//                } catch (ParseException e) {
//                    e.printStackTrace();
//                }
            }
            if ("1".equals(workingConditionCalculationLogModel.getIsCalculate())) {
                workingConditionCalculationLogModel.setIsCalculate("2");
            }
            ;
            if (i == (list.size() - 2)) {
                workingConditionCalculationLogModel.setIsCalculate("3");
            }
            workingConditionCalculationLogModel.setCalTime(nowPoint.getTime());
            workingConditionCalculationLogService.updateById(workingConditionCalculationLogModel);

        }
    }

    /**
     * 泵压预警重新计算
     *
     * @param jh        井号
     * @param startTime 开始时间
     * @param endTime   结束时间
     */
    @SneakyThrows
    @Async("workingConditionCalculationTaskExecutor")
    public void pumpingPressureWarningCalculation(String jh, String startTime, String endTime, WorkingConditionCalculationLog workingConditionCalculationLogModel, String isDelRedis) {
        Thread currentThread = Thread.currentThread();
        String threadName = currentThread.getName();
        long threadId = currentThread.getId();
        Thread.State threadState = currentThread.getState();
        System.out.println("当前线程名称: " + threadName);
        System.out.println("当前线程ID: " + threadId);
        System.out.println("当前线程状态: " + threadState);
        List<WellDrilling> list = queryList(jh, startTime, endTime);
        String redisHasKey = jh + "_pumping_pressure_recalculate";
        QueryWrapper<PumpingPressureWarnLog> pumpingPressureWarnLogQueryWrapper = new QueryWrapper<>();
        pumpingPressureWarnLogQueryWrapper.apply("WARN_TIME <= to_date({0},'yyyy-mm-dd hh24:mi:ss')", endTime);
        pumpingPressureWarnLogQueryWrapper.apply("WARN_TIME >= to_date({0},'yyyy-mm-dd hh24:mi:ss')", startTime);
        pumpingPressureWarnLogService.remove(pumpingPressureWarnLogQueryWrapper);
        if ("0".equals(isDelRedis)) {
            redisUtil.del(redisHasKey);
        }
        for (int i = 0; i < list.size() - 1; i++) {
            WellDrilling nowPoint = list.get(i);
            BigDecimal nowSpp = nowPoint.getSpp().setScale(2, RoundingMode.HALF_UP);
            List<YjPumpingWarnValue> warnValueDatas = getPumpWarnValues(jh);
            YjPumpingWarnValue warnValueData = filterPumpWarnData(nowSpp, warnValueDatas);
            log.info("筛选后的阈值数据:{}",warnValueData);
            BigDecimal pumpingWarnValue = warnValueData.getPumpingWarnValue();
            BigDecimal sppWarnValue = warnValueData.getSppWarnValue();
            BigDecimal warnBitdepth = warnValueData.getBitdepth();
            BigDecimal bitdepth = nowPoint.getBitdepth();
            log.info("当前点井深{};阈值井深:{},时间:{}", nowPoint.getBitdepth(), warnBitdepth, nowPoint.getTime());
            int bitdepthThreshold = bitdepth.compareTo(warnBitdepth);
            if (bitdepthThreshold >= 0) {
                pumpingPressureService.pumpingPressureCalculate(nowPoint, redisHasKey, pumpingWarnValue, sppWarnValue);
                //pumpingPressureService.pumpingPressureCalculate(nowPoint, redisHasKey);
            }
            if ("1".equals(workingConditionCalculationLogModel.getIsCalculate())) {
                workingConditionCalculationLogModel.setIsCalculate("2");
            }
            ;
            if (i == (list.size() - 2)) {
                workingConditionCalculationLogModel.setIsCalculate("3");
            }
            workingConditionCalculationLogModel.setCalTime(nowPoint.getTime());
            workingConditionCalculationLogService.updateById(workingConditionCalculationLogModel);
        }
    }

    /**
     * 扭矩预警重新计算
     *
     * @param jh        井号
     * @param startTime 开始时间
     * @param endTime   结束时间
     */
    @SneakyThrows
    @Async("workingConditionCalculationTaskExecutor")
    public void torqueWarningCalculation(String jh, String startTime, String endTime, WorkingConditionCalculationLog workingConditionCalculationLogModel, String isDelRedis) {
        Thread currentThread = Thread.currentThread();
        String threadName = currentThread.getName();
        long threadId = currentThread.getId();
        Thread.State threadState = currentThread.getState();
        System.out.println("当前线程名称: " + threadName);
        System.out.println("当前线程ID: " + threadId);
        System.out.println("当前线程状态: " + threadState);
        List<WellDrilling> list = queryList(jh, startTime, endTime);
        QueryWrapper<TorqueWarnLog> torqueWarnLogQueryWrapper = new QueryWrapper<>();
        torqueWarnLogQueryWrapper.apply("WARN_START_TIME <= to_date({0},'yyyy-mm-dd hh24:mi:ss')", endTime);
        torqueWarnLogQueryWrapper.apply("WARN_START_TIME >= to_date({0},'yyyy-mm-dd hh24:mi:ss')", startTime);
        torqueWarnLogService.remove(torqueWarnLogQueryWrapper);
        String redisHasKey = jh + "_torque_recalculate";
        if ("0".equals(isDelRedis)) {
            redisUtil.del(redisHasKey);
        }

        for (int i = 0; i < list.size() - 1; i++) {
            WellDrilling nowPoint = list.get(i);
            BigDecimal torque = nowPoint.getTorque();
            List<YjTorqueWarnValue> warnValueDatas = getWarnValues(jh);
            YjTorqueWarnValue yjTorqueWarnValue = filterWarnData(torque, warnValueDatas);
            BigDecimal warnBitdepth = yjTorqueWarnValue.getBitdepth();
            //均值预警值
            BigDecimal varianceWarnValue = yjTorqueWarnValue.getVarianceWarnValue();
            //波动值预警值
            BigDecimal fluctuateWarn = yjTorqueWarnValue.getFluctuateWarn();
            BigDecimal bitdepth = nowPoint.getBitdepth();
            log.info("当前点井深{};阈值井深:{},时间:{}", nowPoint.getBitdepth(), warnBitdepth, nowPoint.getTime());
            int bitdepthThreshold = bitdepth.compareTo(warnBitdepth);
            if (bitdepthThreshold >= 0) {
//                BigDecimal varianceWarnValue = BigDecimal.valueOf(0.5);
//                BigDecimal fluctuateWarn = BigDecimal.valueOf(0.5);
                torqueService.torqueCalculate(nowPoint, redisHasKey, varianceWarnValue, fluctuateWarn);
            } else {
                log.info("钻头位置小于500米;当前点井深{};时间:{}", nowPoint.getBitdepth(), nowPoint.getTime());
            }
            if ("1".equals(workingConditionCalculationLogModel.getIsCalculate())) {
                workingConditionCalculationLogModel.setIsCalculate("2");
            }
            ;
            if (i == (list.size() - 2)) {
                workingConditionCalculationLogModel.setIsCalculate("3");
            }
            workingConditionCalculationLogModel.setCalTime(nowPoint.getTime());
            workingConditionCalculationLogService.updateById(workingConditionCalculationLogModel);
        }
    }

    public List<WellDrilling> queryList(String jh, String startTime, String endTime) {
        QueryWrapper<WellDrilling> wrapper = new QueryWrapper<>();
        wrapper.apply("TO_DATE(SUBSTR(TIME, 1, 19), 'yyyy-MM-dd HH24:mi:ss') <= to_date({0},'yyyy-mm-dd hh24:mi:ss')", endTime);
        wrapper.apply("TO_DATE(SUBSTR(TIME, 1, 19), 'yyyy-MM-dd HH24:mi:ss') >= to_date({0},'yyyy-mm-dd hh24:mi:ss')", startTime);
        wrapper.orderByAsc("TIME");
        String jhTableName = CommonUtils.getJhTableName(jh);
        return wellDrillingService.selectListOrcale(jhTableName, wrapper);
    }

//    public void calWorkingCondition(String jhTableName,List<WellDrilling> list,OperatingConditionThreshold one,WorkingConditionCalculationLog workingConditionCalculationLogModel){
//
//    }


    public String warnTorqueValueKey = "_torque_warn_value_con_key";

    public List<YjTorqueWarnValue> getWarnValues(String jh) {
        String warnKey = jh + warnTorqueValueKey;
        boolean isWarnKey = redisUtil.hasKey(warnKey);
        List<YjTorqueWarnValue> list = new ArrayList<>();
        if (isWarnKey) {
            Object o = redisUtil.get(warnKey);
            list = CommonUtil.castList(o, YjTorqueWarnValue.class);
        } else {
            list = torqueWarnValueService.getListByJh(jh);
            redisUtil.set(warnKey, list);
        }
        return list;
    }

    /**
     * @param nowTorque
     * @param warnValueDatas
     */
    public YjTorqueWarnValue filterWarnData(BigDecimal nowTorque, List<YjTorqueWarnValue> warnValueDatas) {
        AtomicReference<YjTorqueWarnValue> model = new AtomicReference<>(new YjTorqueWarnValue());
        log.info("筛选的阈值数组:{}", warnValueDatas);
        log.info("当前点扭矩：{}", nowTorque);
        warnValueDatas.forEach(item -> {
            BigDecimal torqueMin = item.getTorqueMin();
            BigDecimal torqueMax = item.getTorqueMax();
            Boolean minIsZero = torqueMin.compareTo(BigDecimal.ZERO) <= 0;
            boolean maxIsZero = torqueMax.compareTo(BigDecimal.ZERO) > 0;
            boolean isMin = nowTorque.compareTo(torqueMin) > 0;
            boolean isMax = nowTorque.compareTo(torqueMax) <= 0;
            log.info("最小值是否小于等于0：{}", minIsZero);
            log.info("最大值是否大于0：{}", maxIsZero);
            log.info("是否小于等于最大值：{};最大值：{}", isMax, torqueMax);
            log.info("是否大于最小值：{};最小值：{}", isMin, torqueMin);
            if (minIsZero) {
                model.set(warnValueDatas.get(0));
            } else {
                if (maxIsZero) {
                    if (isMin && isMax) {
                        model.set(item);
                    }
                } else {
                    if (isMin) {
                        model.set(item);
                    }
                }
            }
        });
        log.info("筛选的阈值结果:{}", model.get());
        return model.get();
    }

    public String warnPumpValueKey = "_pump_warn_value_con_key";

    public List<YjPumpingWarnValue> getPumpWarnValues(String jh) {
        String warnKey = jh + warnPumpValueKey;
        boolean isWarnKey = redisUtil.hasKey(warnKey);
        List<YjPumpingWarnValue> list = new ArrayList<>();
        if (isWarnKey) {
            Object o = redisUtil.get(warnKey);
            list = CommonUtil.castList(o, YjPumpingWarnValue.class);
        } else {
            list = pumpingWarnValueService.getListByJh(jh);
            redisUtil.set(warnKey, list);
        }
        return list;
    }


    /**
     * @param nowSpp
     * @param warnValueDatas
     */
    public YjPumpingWarnValue filterPumpWarnData(BigDecimal nowSpp, List<YjPumpingWarnValue> warnValueDatas) {
        AtomicReference<YjPumpingWarnValue> model = new AtomicReference<>(new YjPumpingWarnValue());
        log.info("当前点立压：{}", nowSpp);
        warnValueDatas.forEach(item -> {
            BigDecimal pumpingMin = item.getPumpingMin();
            BigDecimal pumpingMax = item.getPumpingMax();
            Boolean minIsZero = pumpingMin.compareTo(BigDecimal.ZERO) <= 0;
            Boolean maxIsZero = pumpingMax.compareTo(BigDecimal.ZERO) > 0;
            boolean isMin = nowSpp.compareTo(pumpingMin) > 0;
            boolean isMax = nowSpp.compareTo(pumpingMax) <= 0;
            log.info("最小值是否小于等于0：{}", minIsZero);
            log.info("最大值是否大于0：{}", maxIsZero);
            log.info("是否小于等于最大值：{};最大值：{}", isMax, pumpingMax);
            log.info("是否大于最小值：{};最小值：{}", isMin, pumpingMin);
            if (minIsZero) {
                model.set(warnValueDatas.get(0));
            } else {
                if (maxIsZero) {
                    if (isMin && isMax) {
                        model.set(item);
                    }
                } else {
                    if (isMin) {
                        log.info("最小值:{}",pumpingMin);
                        log.info("阈值数据:{}",item);
                        model.set(item);
                    }
                }
            }
        });
        YjPumpingWarnValue yjPumpingWarnValue = model.get();
        return yjPumpingWarnValue;
    }

}
