package com.cn.tianji.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cn.tianji.common.CommonUtils;
import com.cn.tianji.common.RedisUtil;
import com.cn.tianji.dto.ConditionDeterminationDto;
import com.cn.tianji.entity.OperatingConditionThreshold;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.entity.WellDrillingSICP;
import com.cn.tianji.entity.WellName;
import com.cn.tianji.service.*;
import com.cn.tianji.util.CommonUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class PullSICPDataServiceImpl implements PullSICPDataService {

    @Resource
    private PullDataSICPService pullDataSICPService;

    @Resource
    private WlwmroDataService wlwmroDataService;

    @Resource
    private WellDrillingService wellDrillingService;

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private OperatingConditionThresholdService operatingConditionThresholdService;

    @Resource
    private ConditionIdentificationService conditionIdentificationService;

    private String catchName = "_Operating";

    @Resource
    private SendWellDrillingMessageService sendWellDrillingMessageService;

    SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    /**
     * 更新井号
     * 1.获取当天上线井号
     * 2.排除来源为胜利的井号
     * 3.获取本地库中存的井数据
     * 4.根据井号判断该井表是否创建
     * 5.
     */

    @Override
    public void updateJh() {
        List<WellName> allJhSicp = pullDataSICPService.getAllJhSicp();
        List<Map<String, String>> allJh = wlwmroDataService.getAllJh();
        List<String> jhs = new ArrayList<>();
        List<String> hhjhs = new ArrayList<>();
        allJhSicp.forEach(item -> {
            String wellName = item.getWellName();
            boolean isMatch = allJh.stream().anyMatch(map -> map.containsValue(wellName));
            if (!isMatch) {
                jhs.add(wellName);
            } else {
                hhjhs.add(wellName);
            }
        });
        log.info("要采集的井号:{}", jhs);
        log.info("黄河的井:{}", hhjhs);
        List<String> createdJhTable = createJhOperatingCatch();
        jhs.forEach(item -> {
            boolean isCreateTable = !createdJhTable.contains(item);
            if (isCreateTable) {
                log.info("需要创建的井表:{}", item);
            }
        });

    }

    public List<String> createJhOperatingCatch() {
        List<String> jhList = wellDrillingService.tableNames();
        jhList.remove("SYNC_RECORD");
        jhList.remove("WELL_MATH_TIME");
        jhList.remove("WELL_NEW_TABLE");
        jhList.remove("WELL_MATH_UP_TIME");
        jhList.remove("DRILLING_CONDITIONS_LOG");
        return (List<String>) jhList.stream().map(it -> "\"" + it + "\"").collect(Collectors.toList());
    }


    public void pullSicpData(List<String> jhList) {
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        for (String jh : jhList) {
            String saveJh = "LTD_" + jh.replace('-', '_').toUpperCase().trim();
            String selectJh = "\"" + saveJh + "\"";
            String lastDate = wellDrillingService.getLastDateByJh(selectJh);
            log.info("采集井号:{}", jh);
            LocalDateTime formatTime = LocalDateTime.parse(lastDate, df);
            // 获取当前时间
            LocalDateTime now = LocalDateTime.now();
            Duration duration = Duration.between(formatTime, now);
            long timeDiff = duration.toMinutes();
            // 判断时间差是否小于半小时（3600000毫秒或60分钟）
            boolean isLessThanOneHour = timeDiff < 30;
            if (isLessThanOneHour) {
                boolean isLessThanThreeMin = timeDiff < 3;
                if (isLessThanThreeMin) {

                }
            } else {
                //超过半小时则以当前时间为准往前推3分钟
            }
        }
        ;
    }

    @Override
    public void updTableByJhs(List<String> jhs){
        List<Map<String, String>> allJh = wlwmroDataService.getAllJh();
        jhs.forEach(item->{
            String saveJh = "LTD_" + item.replace('-', '_').toUpperCase().trim();
            boolean isMatch = allJh.stream().anyMatch(map -> map.containsValue(saveJh));
            if(!isMatch){
                wellDrillingService.createTable(saveJh, System.currentTimeMillis());
            }
        });
    }

    @Override
    public void pullData(List<String> jhs){
        jhs.forEach(item->{
            String saveJh = "LTD_" + item.replace('-', '_').toUpperCase().trim();
            String lastDate = wellDrillingService.getLastDateByJh(saveJh);
            if (!StringUtils.isEmpty(lastDate)) {
                lastDate = lastDate.contains(".") ? lastDate.substring(0, lastDate.indexOf(".")) : lastDate;
            }else{
                Date nowDate=new Date();
                Date specifiedTime = CommonUtils.getSpecifiedTime(nowDate, Calendar.MINUTE, -3);
                lastDate= sdf.format(specifiedTime);
            }
            DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            LocalDateTime now = LocalDateTime.now();
            LocalDateTime formatTime = LocalDateTime.parse(lastDate, df);
            Duration duration = Duration.between(formatTime, now);
            long timeDiff = duration.toMinutes();
            if(timeDiff>1){
                pullDataByJh(item,lastDate);
            }
        });
    }


    public void pullDataByJh(String jh, String time) {
        String saveJh = "LTD_" + jh.replace('-', '_').toUpperCase().trim();
        log.info("采集井号:{};采集时间:{}",jh,time);
        List<WellDrillingSICP> sicpWellDataByJhTime = pullDataSICPService.getSicpWellDataByJhTime(jh, time);
        log.info("数据长度:{}",sicpWellDataByJhTime.size());
        if(sicpWellDataByJhTime.size()>0){
        List<WellDrilling> saveList = sicpToLtdData(sicpWellDataByJhTime);
        conditionDetermination(saveList,saveJh,jh,time);
        }
    }

    public void conditionDetermination(List<WellDrilling> saveList,String tableName, String jh,String startTime){
        List<WellDrilling> startBeforData = getStartBeforData(jh, startTime);
        List<String> startBeforTime1 = startBeforData.stream().map(WellDrilling::getTime).collect(Collectors.toList());
        log.info("首次缓存的数据:{}",startBeforTime1);
        saveList.sort((a, b) -> a.getTime().compareTo(b.getTime()));
        for (int i=0;i<saveList.size();i++){
//            List<WellDrilling> conList=new ArrayList<>();
            WellDrilling model = saveList.get(i);
//            conList=startBeforData;
            startBeforData.add(model);
            //log.info("参考数据:{}",startBeforData);
            OperatingConditionThreshold operatingConditionThreshold = getOperatingCatchByJh(jh);
            log.info("识别时间:{}",model.getTime());
            List<String> collect = startBeforData.stream().map(WellDrilling::getTime).collect(Collectors.toList());
            log.info("识别的数据:{}",collect);
            ConditionDeterminationDto conditionDeterminationDto = conditionIdentificationService.conditionDetermination(startBeforData, operatingConditionThreshold);
            saveList.get(i).setOperatingMode(conditionDeterminationDto.getOperatingMode());
            saveList.get(i).setMotion(conditionDeterminationDto.getMotion());
            //log.info("识别的数据:{}",saveList.get(i));
            sendWellDrillingMessageService.sendMessage(conditionDeterminationDto.getMotion(),  saveList.get(i),  saveList.get(i).getJh());
            startBeforData.remove(0);
//            startBeforData.add(model);
            List<String> startBeforTime = startBeforData.stream().map(WellDrilling::getTime).collect(Collectors.toList());
            log.info("缓存的数据:{}",startBeforTime);
        }
//        redisUtil.del(jh+"_beforData_condition");
//        redisUtil.set(jh+"_beforData_condition",startBeforData);
//        log.info("缓存数据:{}",startBeforData);
       // log.info("插入的数据:{}",saveList);
        String selectJh = "\"" + tableName + "\"";
        int i = wellDrillingService.insertTableBatch(selectJh, saveList);
        log.info("井号;{};插入数据数量:{}", selectJh, i);
    }

    public OperatingConditionThreshold getOperatingCatchByJh(String jh) {
        String redisKsy = jh + catchName;
        OperatingConditionThreshold catchData = new OperatingConditionThreshold();
        boolean isHaveCatch = redisUtil.hasKey(redisKsy);
        if (isHaveCatch) {
            catchData = JSONObject.toJavaObject(JSONObject.parseObject(JSON.toJSONString(redisUtil.get(redisKsy))), OperatingConditionThreshold.class);
        } else {
            // catchData = setCreating();
            getOp(jh);
        }
        return catchData;
    }




    public void getOp(String jh){
        String redisKsy = jh + catchName;
        QueryWrapper<OperatingConditionThreshold> wrapper=new QueryWrapper<>();
        wrapper.lambda().eq(OperatingConditionThreshold::getJh,jh);
        List<OperatingConditionThreshold> list = operatingConditionThresholdService.list(wrapper);
        if(list.size()>0){
            OperatingConditionThreshold operatingConditionThreshold = list.get(0);
            redisUtil.del(redisKsy);
            redisUtil.set(redisKsy,operatingConditionThreshold);
        }else{
            OperatingConditionThreshold saveModel = setCreating();
            saveModel.setJh(jh);
            operatingConditionThresholdService.save(saveModel);
        }
    }

    public OperatingConditionThreshold setCreating() {
        OperatingConditionThreshold operatingConditionThreshold = new OperatingConditionThreshold();
        operatingConditionThreshold.setJh("通用");
        operatingConditionThreshold.setApproachingSurface(BigDecimal.valueOf(30));
        operatingConditionThreshold.setWoh(BigDecimal.valueOf(210));
        operatingConditionThreshold.setOnBottom(BigDecimal.valueOf(5));
        operatingConditionThreshold.setIsRotationOff(BigDecimal.valueOf(5));
        operatingConditionThreshold.setBlockldle(BigDecimal.valueOf(0));
        operatingConditionThreshold.setNoFlow(BigDecimal.valueOf(15));
        operatingConditionThreshold.setPressureZero(BigDecimal.valueOf(5));
        operatingConditionThreshold.setBitMovingUp(BigDecimal.valueOf(0));
        operatingConditionThreshold.setBlockMovingUp(BigDecimal.valueOf(0));
        operatingConditionThreshold.setFarBottom(BigDecimal.valueOf(30));
        return operatingConditionThreshold;
    }

    @SneakyThrows
    public  List<WellDrilling> getStartBeforData(String jh, String startTime){
        String redisKey=jh+"_beforData_condition";
        boolean isHaveBeforData = redisUtil.hasKey(redisKey);
        List<WellDrilling> beforData=new ArrayList<>();
        if(isHaveBeforData){
            Object listObject = redisUtil.get(redisKey);
            String listStr = CommonUtil.convertToString(listObject);
            ObjectMapper objectMapper = new ObjectMapper();
            try {
                // 将JSON字符串解析为List<MyEntity>
                beforData= objectMapper.readValue(listStr, objectMapper.getTypeFactory().constructCollectionType(List.class, WellDrilling.class));
            } catch (IOException e) {
                e.printStackTrace();
            }
        }else{
            log.info("首次获取数据");
            Date parse = sdf.parse(startTime);
            Date specifiedTime = CommonUtils.getSpecifiedTime(parse, Calendar.MINUTE, -3);
            List<WellDrillingSICP> sicpWellDataByJhTime = pullDataSICPService.getSicpWellDataByJhTime(jh, sdf.format(specifiedTime));
            List<WellDrilling> startBeforData =  sicpToLtdData(sicpWellDataByJhTime);
            List<WellDrilling> wellDrillings = filterData(startBeforData, parse);
            List<String> startBeforTime = wellDrillings.stream().map(WellDrilling::getTime).collect(Collectors.toList());
            log.info("筛选后的数据:{}",startBeforTime);
            beforData.add(wellDrillings.get(wellDrillings.size()-3));
            beforData.add(wellDrillings.get(wellDrillings.size()-2));
        }
        log.info("beforData数据长度{}",beforData.size());
       // log.info("最终的结果数据:{}",beforData);
       return beforData;
    }

    public List<WellDrilling> filterData( List<WellDrilling> list,Date time){
        List<WellDrilling> res=new ArrayList<>();
        list.forEach(item->{
            String timec = item.getTime();
            timec= timec.contains(".") ? timec.substring(0, timec.indexOf(".")) : timec;
            Date parse =new Date();
            try {
                 parse = sdf.parse(timec);
            } catch (ParseException e) {
                e.printStackTrace();
            }
            boolean before = time.before(parse);
            if(!before){
                res.add(item);
            }
        });
       // log.info("筛选的结果数据:{}",res);

        res.remove(res.size()-1);
        res.sort((a, b) -> a.getTime().compareTo(b.getTime()));
        res.forEach(items->log.info("时间:{}",items.getTime()));
        return res;
    }

    public  List<WellDrilling> sicpToLtdData(List<WellDrillingSICP> sicpList) {
        List<WellDrilling> saveList = new ArrayList<>();
        sicpList.forEach(item -> {
            WellDrilling model = sicpToLtd(item);
            saveList.add(model);
        });
        return saveList ;
    }

    public WellDrilling sicpToLtd(WellDrillingSICP item) {
        WellDrilling model = new WellDrilling();
        model.setJh(item.getJh());
        model.setTid(item.getTIME());
        model.setTime(item.getTIME());
        model.setId(BigDecimal.valueOf(-999.25));
        try {
            model.setDepth(CommonUtil.convertStringToBigDecimal(item.getDMEA()));
            model.setVdepth(CommonUtil.convertStringToBigDecimal(item.getDVER()));
            model.setLagtime(CommonUtil.convertStringToBigDecimal(item.getTRTT()));
            model.setLagstrokes(CommonUtil.convertStringToBigDecimal(item.getLSTK()));
            model.setBitdepth(CommonUtil.convertStringToBigDecimal(item.getDBTM()));
            model.setKellyin(CommonUtil.convertStringToBigDecimal(item.getKELLYIN()));
            model.setHkh(CommonUtil.convertStringToBigDecimal(item.getBPOS()));
            model.setHks(CommonUtil.convertStringToBigDecimal(item.getBSPD()));
            model.setWoh(CommonUtil.convertStringToBigDecimal(item.getHKLA()));
            model.setRpm(CommonUtil.convertStringToBigDecimal(item.getRPMA()));

            model.setWob(CommonUtil.convertStringToBigDecimal(item.getWOBA()));
            BigDecimal tqa= CommonUtil.convertStringToBigDecimal(item.getTQA());
            if(tqa.compareTo(BigDecimal.valueOf(0.0))>0){
                model.setTorque(tqa);
            }else{
                model.setTorque(CommonUtil.convertStringToBigDecimal(item.getTTQA()));
            }
            model.setTdTorque(CommonUtil.convertStringToBigDecimal(item.getTTQA()));
            model.setSpp(CommonUtil.convertStringToBigDecimal(item.getSPPA()));
            model.setWhp(CommonUtil.convertStringToBigDecimal(item.getCHKP()));
            model.setPump1(CommonUtil.convertStringToBigDecimal(item.getSPM1()));
            model.setPump2(CommonUtil.convertStringToBigDecimal(item.getSPM2()));
            model.setPump3(CommonUtil.convertStringToBigDecimal(item.getSPM3()));
            model.setPumptotal(CommonUtil.convertStringToBigDecimal(item.getTSPM()));

            model.setStrokes1(CommonUtil.convertStringToBigDecimal(item.getSTROKES1()));
            model.setStrokes2(CommonUtil.convertStringToBigDecimal(item.getSTROKES2()));
            model.setStrokes3(CommonUtil.convertStringToBigDecimal(item.getSTROKES3()));
            model.setStrokestotal(CommonUtil.convertStringToBigDecimal(item.getSTKC()));
            model.setVopInst(CommonUtil.convertStringToBigDecimal(item.getVOP_INST()));
            model.setVopMavg(CommonUtil.convertStringToBigDecimal(item.getVOP_MAVG()));
            model.setFlowin(CommonUtil.convertStringToBigDecimal(item.getMFIA()));
            model.setFlowout(CommonUtil.convertStringToBigDecimal(item.getMFOA()));
            model.setFlowoutpercent(CommonUtil.convertStringToBigDecimal(item.getMFOP()));
            model.setMwin(CommonUtil.convertStringToBigDecimal(item.getMDIA()));

            model.setMwout(CommonUtil.convertStringToBigDecimal(item.getMDOA()));
            model.setTempin(CommonUtil.convertStringToBigDecimal(item.getMTIA()));
            model.setTempout(CommonUtil.convertStringToBigDecimal(item.getMTOA()));
            model.setCondin(CommonUtil.convertStringToBigDecimal(item.getMCOA()));
            model.setCondout(CommonUtil.convertStringToBigDecimal(item.getMCIA()));
            model.setPit1(CommonUtil.convertStringToBigDecimal(item.getTV01()));
            model.setPit2(CommonUtil.convertStringToBigDecimal(item.getTV02()));
            model.setPit3(CommonUtil.convertStringToBigDecimal(item.getTV03()));
            model.setPit4(CommonUtil.convertStringToBigDecimal(item.getTV04()));
            model.setPit5(CommonUtil.convertStringToBigDecimal(item.getTV05()));
            model.setPit6(CommonUtil.convertStringToBigDecimal(item.getTV06()));
            model.setPit7(CommonUtil.convertStringToBigDecimal(item.getTV07()));
            model.setPit8(CommonUtil.convertStringToBigDecimal(item.getTV08()));
            model.setPit9(CommonUtil.convertStringToBigDecimal(item.getTV09()));
            model.setPit10(CommonUtil.convertStringToBigDecimal(item.getTV10()));
            model.setPit11(CommonUtil.convertStringToBigDecimal(item.getTV11()));
            model.setPit12(CommonUtil.convertStringToBigDecimal(item.getTV12()));
            model.setPittotal(CommonUtil.convertStringToBigDecimal(item.getTVT()));

            model.setC1(CommonUtil.convertStringToBigDecimal(item.getMETH()));
            model.setC2(CommonUtil.convertStringToBigDecimal(item.getETH()));
            model.setC3(CommonUtil.convertStringToBigDecimal(item.getPRP()));
            model.setIc4(CommonUtil.convertStringToBigDecimal(item.getIBUT()));
            model.setNc4(CommonUtil.convertStringToBigDecimal(item.getNBUT()));
            model.setIc5(CommonUtil.convertStringToBigDecimal(item.getIPEN()));
            model.setNc5(CommonUtil.convertStringToBigDecimal(item.getNPEN()));
            model.setEc5(CommonUtil.convertStringToBigDecimal(item.getEPEN()));
            model.setIc6(CommonUtil.convertStringToBigDecimal(item.getIHEX()));
            model.setNc6(CommonUtil.convertStringToBigDecimal(item.getNHEX()));

            model.setStat(BigDecimal.valueOf(-999.25));
            model.setLagdepth(BigDecimal.valueOf(-999.25));
            model.setBitoff(BigDecimal.valueOf(-999.25));
            model.setWohCalc(BigDecimal.valueOf(-999.25));
            model.setFloatWeight(BigDecimal.valueOf(-999.25));
            model.setRopInst(BigDecimal.valueOf(-999.25));
            model.setRopMavg(BigDecimal.valueOf(-999.25));
            model.setEcd(BigDecimal.valueOf(-999.25));
            model.setTriptandvol(BigDecimal.valueOf(-999.25));
            model.setActpittotal(BigDecimal.valueOf(-999.25));
            model.setGainlossvol(BigDecimal.valueOf(-999.25));
            model.setTg(BigDecimal.valueOf(-999.25));
            model.setCo2(BigDecimal.valueOf(-999.25));
            model.setH2(BigDecimal.valueOf(-999.25));
            model.setH2s1(BigDecimal.valueOf(-999.25));
            model.setH2s2(BigDecimal.valueOf(-999.25));
            model.setH2s3(BigDecimal.valueOf(-999.25));
            model.setH2s4(BigDecimal.valueOf(-999.25));
            model.setH2s5(BigDecimal.valueOf(-999.25));
            model.setH2s6(BigDecimal.valueOf(-999.25));
            model.setH2s7(BigDecimal.valueOf(-999.25));
            model.setH2s8(BigDecimal.valueOf(-999.25));

            model.setStringlen(BigDecimal.valueOf(-999.25));
            model.setCurpipe(BigDecimal.valueOf(-999.25));
            model.setBitno(BigDecimal.valueOf(-999.25));
            model.setBitsize(BigDecimal.valueOf(-999.25));
            model.setBittype(BigDecimal.valueOf(-999.25));
            model.setBitrun(BigDecimal.valueOf(-999.25));
            model.setBittime(BigDecimal.valueOf(-999.25));
            model.setBitdrilltime(BigDecimal.valueOf(-999.25));
            model.setBitcost(BigDecimal.valueOf(-999.25));
            model.setTotaldays(BigDecimal.valueOf(-999.25));

            model.setStandsdone(BigDecimal.valueOf(-999.25));
            model.setStandstogo(BigDecimal.valueOf(-999.25));
            model.setTripoverpull(BigDecimal.valueOf(-999.25));
            model.setDc(BigDecimal.valueOf(-999.25));
            model.setSigma(BigDecimal.valueOf(-999.25));
            model.setFree95(BigDecimal.valueOf(-999.25));
            model.setFree96(BigDecimal.valueOf(-999.25));
            model.setFree97(BigDecimal.valueOf(-999.25));
            model.setFree98(BigDecimal.valueOf(-999.25));
            model.setFree99(BigDecimal.valueOf(-999.25));
            model.setFree100(BigDecimal.valueOf(-999.25));
            model.setFree101(BigDecimal.valueOf(-999.25));
            model.setFree102(BigDecimal.valueOf(-999.25));
            model.setFree103(BigDecimal.valueOf(-999.25));
            model.setFree104(BigDecimal.valueOf(-999.25));
            model.setFree105(BigDecimal.valueOf(-999.25));
            model.setFree106(BigDecimal.valueOf(-999.25));
            model.setFree107(BigDecimal.valueOf(-999.25));
            model.setFree108(BigDecimal.valueOf(-999.25));
            model.setFree109(BigDecimal.valueOf(-999.25));
            model.setFree110(BigDecimal.valueOf(-999.25));
            model.setFree111(BigDecimal.valueOf(-999.25));
            model.setFree112(BigDecimal.valueOf(-999.25));
            model.setFree113(BigDecimal.valueOf(-999.25));
            model.setFree114(BigDecimal.valueOf(-999.25));
            model.setFree115(BigDecimal.valueOf(-999.25));
            model.setFree116(BigDecimal.valueOf(-999.25));
            model.setFree117(BigDecimal.valueOf(-999.25));
            model.setFree118(BigDecimal.valueOf(-999.25));
            model.setFree119(BigDecimal.valueOf(-999.25));
            model.setFree120(BigDecimal.valueOf(-999.25));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return model;
    }

    @Override
    public void createTable(List<String> jhs){
        pullDataSICPService.createTables(jhs);
    }
}
