package com.cn.tianji.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cn.tianji.common.CommonUtils;
import com.cn.tianji.common.HttpUtil;
import com.cn.tianji.common.RedisUtil;
import com.cn.tianji.entity.*;
import com.cn.tianji.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class PullWellDrillingDataServiceImpl implements PullWellDrillingDataService {
    @Resource
    private WellDrillingService wellDrillingService;

    @Resource
    private WorkingConditionCalculationLogService workingConditionCalculationLogService;

    @Resource
    private SendWellDrillingMessageService sendWellDrillingMessageService;


    @Resource
    private HookLoadWarnLogService hookLoadWarnLogService;

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private RestTemplate restTemplate;

    @Resource
    private WlwmroDataService wlwmroDataService;

    @Resource
    private AjhService ajhService;


    @Override
    public void updateAjh(){
        List<String> jhList = wellDrillingService.tableNames();
        jhList.remove("SYNC_RECORD");
        jhList.remove("WELL_MATH_TIME");
        jhList.remove("WELL_NEW_TABLE");
        jhList.remove("WELL_MATH_UP_TIME");
        jhList.remove("DRILLING_CONDITIONS_LOG");
        jhList.forEach(item->{
            String ltd_ = item.replace("LTD_", "");
            String jh = ltd_.replace("_", "-");
            ajhService.saveData(jh);
        });
    }


    @Override
    @Async("pullDataTaskExecutor")
    public void pullData(String workingConditionCalculationLogId) {
        WorkingConditionCalculationLog workingConditionCalculationLogModel = workingConditionCalculationLogService.getById(workingConditionCalculationLogId);
        String jh = workingConditionCalculationLogModel.getJh();
        String startTime = workingConditionCalculationLogModel.getStartTime();
        String endTime = workingConditionCalculationLogModel.getEndTime();
        String calculateType = workingConditionCalculationLogModel.getCalculateType();
        String isDelRedis = workingConditionCalculationLogModel.getIsDelRedis();
        List<WellDrilling> list = queryList(jh, startTime, endTime);
        Thread currentThread = Thread.currentThread();
        String threadName = currentThread.getName();
        long threadId = currentThread.getId();
        Thread.State threadState = currentThread.getState();
        System.out.println("当前线程名称: " + threadName);
        System.out.println("当前线程ID: " + threadId);
        System.out.println("当前线程状态: " + threadState);
        if ("1".equals(isDelRedis)) {
            deleteRedisByJh(jh);
        }
        list.forEach(item -> {
            sendWellDrillingMessageService.sendMessage(item.getOperatingMode(), item, item.getJh());
        });
    }

    @Override
    public void pullListData(List<String> listId) {
        List<WorkingConditionCalculationLog> workingConditionCalculationLogList = workingConditionCalculationLogService.listByIds(listId);
        List<List<WellDrilling>> lists = new ArrayList<>();
        int listSize=0;
        for (WorkingConditionCalculationLog model : workingConditionCalculationLogList) {
            String jh = model.getJh();
            String startTime = model.getStartTime();
            String endTime = model.getEndTime();
            List<WellDrilling> list = queryList(jh, startTime, endTime);
            if(listSize<list.size()){
                listSize=list.size();
            }
            lists.add(list);
            deleteRedisByJh(jh);
        }
        sendLists(lists,listSize);
    }

    public void sendLists(List<List<WellDrilling>> lists, Integer listSize) {
        for (int i = 0; i < listSize; i++) {
            sendData(lists, i);
            try {
                // 休眠5秒（5000毫秒）
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                // 处理异常情况
                e.printStackTrace();
            }
        }
    }
    public void sendData(List<List<WellDrilling>> lists, Integer index) {
        List<WellDrilling> sendList = new ArrayList<>();
        lists.forEach(item -> {
            if (index < item.size()) {
                WellDrilling model = item.get(index);
                sendWellDrillingMessageService.sendMessage(model.getOperatingMode(), model, model.getJh());
            }
        });
    }

    public void deleteRedisByJh(String jh) {
        String redisHasKeyHookLoad = jh + "_hook_load_real_time_computing";
        String redisHasKeyPumping = jh + "_pumping_pressure_real_time_computing";
        String redisHasKeyTorque = jh + "_torque_real_time_computing";
        String redisHasKeyTotalPool = jh + "_total_pool_volume_real_time_computing";

        log.info("清除缓存;井:{}",jh);
        redisUtil.del(redisHasKeyHookLoad);
        redisUtil.del(redisHasKeyPumping);
        redisUtil.del(redisHasKeyTorque);
        redisUtil.del(redisHasKeyTotalPool);
        System.out.println("清除缓存完成");
    }

    public List<WellDrilling> queryList(String jh, String startTime, String endTime) {
        QueryWrapper<WellDrilling> wrapper = new QueryWrapper<>();
        wrapper.apply("TO_DATE(SUBSTR(TIME, 1, 19), 'yyyy-MM-dd HH24:mi:ss') <= to_date({0},'yyyy-mm-dd hh24:mi:ss')", endTime);
        wrapper.apply("TO_DATE(SUBSTR(TIME, 1, 19), 'yyyy-MM-dd HH24:mi:ss') >= to_date({0},'yyyy-mm-dd hh24:mi:ss')", startTime);
        wrapper.orderByAsc("TIME");
        String jhTableName = CommonUtils.getJhTableName(jh);
        return wellDrillingService.selectListOrcale(jhTableName, wrapper);
    }



    @Transactional
    @Override
    public void saveWorkingConditionCalculationLog(WorkingConditionCalculationLog workingConditionCalculationLogModel) {
        workingConditionCalculationLogService.save(workingConditionCalculationLogModel);
    }

    @Override
    public  List<HookLoadWarnLog> getWarnLog(String jh, String startTime, String endTime){
        QueryWrapper<HookLoadWarnLog> wrapper=new QueryWrapper<>();
        wrapper.lambda().eq(HookLoadWarnLog::getJh,jh).and(
                wrapper1 -> wrapper1
                        .apply("WARN_TIME >= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", startTime)
                        .apply("WARN_TIME <= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", endTime).or()
                        .apply("MOBIL_DRILLING_TOOLS_START_TIME >= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", startTime)
                        .apply("MOBIL_DRILLING_TOOLS_START_TIME <= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", endTime)
        ).orderByDesc(HookLoadWarnLog::getWarnTime,HookLoadWarnLog::getMobilDrillingToolsStartTime);
        return hookLoadWarnLogService.list(wrapper);
    }

    @Override
    public List<WellName> getAllJhSicp(){
        // 发送POST请求并获取响应结果
        String url = "http://10.68.224.20:15221/v1/structuredData/serviceData/SICPALLJH?app_key=SICP";
        log.info("请求获取井号地址:{}",url);
        JSONObject params=new JSONObject();
        String resultJson = HttpUtil.sendPost(url,"{}");
        log.info("获取的所有井号:{}",resultJson);
        JSONObject obj = JSONObject.parseObject(resultJson);
        String statusCode= obj.getString("statusCode");
        JSONObject data = obj.getJSONObject("data");
        List<WellName> wellNameList=new ArrayList<>();
        if("000".equals(statusCode)){
            JSONArray rows = data.getJSONArray("rows");
            JSONObject o =JSONObject.parseObject(String.valueOf(rows.get(0)));
            String alljh = o.getString("alljh");
            String[] split = alljh.split("、");
            for(String jh:split){
                WellName model=new WellName();
                model.setWellName(jh);
                wellNameList.add(model);
            }
        }
        return wellNameList;
    }

    @Override
    public List<WellDrillingSICP> syncTableData(String jh, String startTime){
        String url = "http://10.68.224.20:15221/v1/structuredData/serviceData/SICP3M?app_key=YF";
        JSONObject params=new JSONObject();
        JSONObject conditionJh=new JSONObject();
        conditionJh.put("condition",jh);
        JSONObject conditionTime=new JSONObject();
        conditionTime.put("condition",startTime);
        params.put("jh",conditionJh);
        params.put("kssj",conditionTime);
        String resultString = HttpUtil.sendPost(url, String.valueOf(params));
        JSONObject resultJson = JSONObject.parseObject(resultString);
        String statusCode= resultJson.getString("statusCode");
        JSONObject data = resultJson.getJSONObject("data");
        log.info("data:{}",data);
        JSONArray rows = data.getJSONArray("rows");
        log.info("rows:{}",rows);
        List<WellDrillingSICP> list=new ArrayList<>();
//        rows.forEach(item->{
            for (Object item:rows){
                // 打印传递的消息
                log.info("item：{}",item);
                WellDrillingSICP model= JSONObject.parseObject(String.valueOf(item), WellDrillingSICP.class);
                String jrtime = model.getJRTIME();
                String time = model.getTIME();
                if(jrtime.contains("+")){
                    int i = jrtime.indexOf("+");
                    String jrtimes=  jrtime.substring(0,i);
                    log.info("截取后的时间数据jrtimes{}",jrtimes);
                    model.setJRTIME(jrtimes);
                }
                if(time.contains("+")){
                    int i = time.indexOf("+");
                    String times=  time.substring(0,i);
                    log.info("截取后的时间数据times{}",times);
                    model.setTIME(times);
                }

                list.add(model);
            }

//        });
        return list;
    }
}
