package com.cn.tianji.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.cn.tianji.common.RedisUtil;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.entity.YjPumpingWarnValue;
import com.cn.tianji.entity.YjTorqueWarnValue;
import com.cn.tianji.service.DrillingOutHookLoadService;
import com.cn.tianji.service.TorqueService;
import com.cn.tianji.service.TorqueWarnService;
import com.cn.tianji.service.YjTorqueWarnValueService;
import com.cn.tianji.util.CommonUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

/**
 * 钩载预警
 *
 * @author Zhao RenJi
 */
@Service
@Slf4j
public class TorqueWarnServiceImpl implements TorqueWarnService {

    @Resource
    private TorqueService torqueService;

    @Resource
    private YjTorqueWarnValueService torqueWarnValueService;

    @Resource
    private RedisUtil redisUtil;

    public String warnValueKey = "_torque_warn_value_key";

    @Override
    public void torqueWarn(WellDrilling nowPoint) {
        String jh = nowPoint.getJh();
        String redisHasKey = jh + "_torque_real_time_computing";
        BigDecimal torque = nowPoint.getTorque();
        List<YjTorqueWarnValue> warnValueDatas = getWarnValues(jh);
        YjTorqueWarnValue yjTorqueWarnValue = filterWarnData(torque, warnValueDatas);
        BigDecimal warnBitdepth = yjTorqueWarnValue.getBitdepth();
        //均值预警值
        BigDecimal varianceWarnValue = yjTorqueWarnValue.getVarianceWarnValue();
        //波动值预警值
        BigDecimal fluctuateWarn = yjTorqueWarnValue.getFluctuateWarn();
        BigDecimal bitdepth = nowPoint.getBitdepth();
        int isIntoButtom = bitdepth.compareTo(warnBitdepth);
        if (isIntoButtom > 0) {
            torqueService.torqueCalculate(nowPoint, redisHasKey, varianceWarnValue, fluctuateWarn);
        } else {
            boolean b = redisUtil.hasKey(redisHasKey);
            if (b) {
                redisUtil.del(redisHasKey);
            }
        }
    }

    public List<YjTorqueWarnValue> getWarnValues(String jh) {
        String warnKey = jh + warnValueKey;
        boolean isWarnKey = redisUtil.hasKey(warnKey);
        List<YjTorqueWarnValue> list = new ArrayList<>();
        if (isWarnKey) {
            Object o = redisUtil.get(warnKey);
            list = CommonUtil.castList(o, YjTorqueWarnValue.class);
        } else {
            list = torqueWarnValueService.getListByJh(jh);
            redisUtil.set(warnKey, list);
        }
        return list;
    }

    /**
     * @param nowTorque
     * @param warnValueDatas
     */
    public YjTorqueWarnValue filterWarnData(BigDecimal nowTorque, List<YjTorqueWarnValue> warnValueDatas) {
        AtomicReference<YjTorqueWarnValue> model = new AtomicReference<>(new YjTorqueWarnValue());
        warnValueDatas.forEach(item -> {
            BigDecimal torqueMin = item.getTorqueMin();
            BigDecimal torqueMax = item.getTorqueMax();
            Boolean minIsZero = torqueMin.compareTo(BigDecimal.ZERO) <= 0;
            boolean maxIsZero = torqueMax.compareTo(BigDecimal.ZERO) > 0;
            boolean isMin = nowTorque.compareTo(torqueMin) > 0;
            boolean isMax = nowTorque.compareTo(torqueMax) <= 0;
            if (minIsZero) {
                model.set(warnValueDatas.get(0));
            } else {
                if (maxIsZero) {
                    if (isMin && isMax) {
                        model.set(item);
                    }
                } else {
                    if (isMin) {
                        model.set(item);
                    }
                }
            }
        });
        return model.get();
    }
}
