package com.cn.tianji.service.impl;


import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.service.CommonConditionDeterminationService;
import com.cn.tianji.service.impl.ConditionDeterminationServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;

/**
 * @author Zhao RenJi
 */
@Service
public class CommonConditionDeterminationServiceImpl  implements CommonConditionDeterminationService {
    @Resource
    private ConditionDeterminationServiceImpl conditionDeterminationService;

    /**
     * 公共判断-是否停转
     *
     * @param nowPoint               当前点数据
     * @param isRotationOffThreshold 停转阈值
     * @return true/false
     */
    @Override
    public Boolean publicRotationOff(WellDrilling nowPoint, BigDecimal isRotationOffThreshold) {
        //判断是否停转
        BigDecimal nowRpmBig = nowPoint.getRpm();
        return conditionDeterminationService.isRotationOff(nowRpmBig, isRotationOffThreshold);
    }

    /**
     * 公共判断-游车是否静止
     *
     * @param nowPoint              当前点数据
     * @param perPoint              上一点点数据
     * @param blockldleThresholdBig 游车静止阈值
     * @return true/false
     */
    @Override
    public Boolean publicBlockldle(WellDrilling nowPoint, WellDrilling perPoint, BigDecimal timeSubBig, BigDecimal blockldleThresholdBig) {
        BigDecimal nowHkhBig = nowPoint.getHkh();
        BigDecimal perHkhBig = perPoint.getHkh();
        return conditionDeterminationService.isBlockldle(nowHkhBig, perHkhBig, timeSubBig, blockldleThresholdBig);
    }


    /**
     * 公共动作判断 判断是否停泵及立压是否为0
     *
     * @param nowPoint                 当前点数据
     * @param noFlowThresholdBig       停泵阈值
     * @param pressureZeroThresholdBig 立压是否为0阈值
     * @return true/false
     */
    @Override
    public Boolean publicPumpAndSpp(WellDrilling nowPoint, BigDecimal noFlowThresholdBig, BigDecimal pressureZeroThresholdBig) {
        BigDecimal pump1Big = nowPoint.getPump1();
        BigDecimal pump2Big = nowPoint.getPump2();
        BigDecimal pump3Big = nowPoint.getPump3();
        BigDecimal sppBig = nowPoint.getSpp();
        //是否停泵
        Boolean isNoFlow = conditionDeterminationService.isNoFlow(pump1Big, pump2Big, pump3Big, noFlowThresholdBig);
        //立压是否为0
        Boolean isPressureZero = conditionDeterminationService.isPressureZero(sppBig, pressureZeroThresholdBig);
        return isNoFlow && isPressureZero;
    }

    /**
     * 公共动作判断 判断钻头/游车是否上行
     *
     * @param nowPoint               当前点数据
     * @param perPoint               上一点点数据
     * @param timeSubBig             时间差
     * @param bitMovingUpThreshold   钻头上行阈值
     * @param blockMovingUpThreshold 游车上行阈值
     * @return true/false
     */
    @Override
    public Boolean publicMovingUp(WellDrilling nowPoint, WellDrilling perPoint, BigDecimal timeSubBig, BigDecimal bitMovingUpThreshold, BigDecimal blockMovingUpThreshold) {
        BigDecimal nowBitdepthBig = nowPoint.getBitdepth();
        BigDecimal perBitdepthBig = perPoint.getBitdepth();
        //钻头是否上行
        Boolean isBitMovingUp = conditionDeterminationService.isBitMovingUp(nowBitdepthBig, perBitdepthBig, timeSubBig, bitMovingUpThreshold);
        BigDecimal nowHkhBig = nowPoint.getHkh();
        BigDecimal perHkhBig = perPoint.getHkh();
        //游车是否上行
        Boolean isBlockMovingUp = conditionDeterminationService.isBlockMovingUp(nowHkhBig, perHkhBig, timeSubBig, blockMovingUpThreshold);
//        isBitMovingUp &&
        return  isBlockMovingUp;
    }
}
