package com.cn.tianji.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.cn.tianji.common.CommonUtils;
import com.cn.tianji.common.DrillingHookLoadCatch;
import com.cn.tianji.common.DrillingOutHookLoadCatch;
import com.cn.tianji.common.RedisUtil;
import com.cn.tianji.entity.HookLoadWarnLog;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.enums.DrillingActionEnum;
import com.cn.tianji.enums.DrillingConditionDeterminationEnum;
import com.cn.tianji.enums.DrillingDownActionEnum;
import com.cn.tianji.enums.DrillingUpActionEnum;
import com.cn.tianji.service.DrillingHookloadService;
import com.cn.tianji.service.HookLoadWarnLogService;
import com.cn.tianji.service.WarnHistoryService;
import com.cn.tianji.service.WellDrillingService;
import jodd.cache.Cache;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author Zhao RenJi
 */
@Slf4j
@Service
public class DrillingHookloadServiceImp implements DrillingHookloadService {

    @Resource
    private WellDrillingService wellDrillingService;

    @Resource
    private WarnHistoryService warnHistoryService;

    @Resource
    private HookLoadWarnLogService hookLoadWarnLogService;

    @Resource
    private RedisUtil redisUtil;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");



    /**
     * 钻进活动钻具钩载预警-使用活动钻具规则
     *
     * @param nowPoint    当前点数据
     * @param redisHasKey 缓存名称
     * @param warnValue   预警阈值
     */
    @Override
    public void hookLoadWarnCalculate(WellDrilling nowPoint, String redisHasKey, BigDecimal warnValue) {
        Date startTime = new Date();

        // BigDecimal warnValue = BigDecimal.valueOf(100);

        String jh = nowPoint.getJh();
        //String redisHasKey = jh + "_drilling_hook_load";
        String motion = nowPoint.getMotion();
        boolean isHave = redisUtil.hasKey(redisHasKey);
        // boolean isWarnAction = CommonUtils.diagnosisWarnAction(DrillingActionEnum.class, motion);
        //是否上提钻具动作
        Boolean isUpAction = CommonUtils.diagnosisWarnAction(DrillingUpActionEnum.class, motion);
        //是否下放钻具动作
        Boolean isDownAction = CommonUtils.diagnosisWarnAction(DrillingDownActionEnum.class, motion);

        //当前点是否为上提/下方钻具动作
        if (isUpAction || isDownAction) {
            //判断当前柱是否存在缓存数据
            if (isHave) {
                DrillingHookLoadCatch hookLoadCatch = JSONObject.toJavaObject(JSONObject.parseObject(JSON.toJSONString(redisUtil.get(redisHasKey))), DrillingHookLoadCatch.class);
                Integer upperDrillingToolSteps = hookLoadCatch.getUpperDrillingToolSteps();
                Integer downDrillingToolSteps = hookLoadCatch.getDownDrillingToolSteps();
                if (isUpAction) {
                    //上提钻具
                    switch (upperDrillingToolSteps) {
                        case 0:
                            List<WellDrilling> upperReferenceData = hookLoadCatch.getUpperReferenceData();
                            upperReferenceData.add(nowPoint);
                            hookLoadCatch.setUpperReferenceData(upperReferenceData);
                            hookLoadCatch.setUpperDrillingToolSteps(1);
                            //当出现上提钻具动作时判断下放钻具钩载
                            hookLoadCatch = downDrillingToolsHandle(hookLoadCatch, warnValue, nowPoint.getJh(), nowPoint.getMotion(), nowPoint.getOperatingMode());
                            break;
                        case 1:
                            List<WellDrilling> upperReferenceData1 = hookLoadCatch.getUpperReferenceData();
                            upperReferenceData1.add(nowPoint);
                            hookLoadCatch.setUpperReferenceData(upperReferenceData1);
                            break;
                        case 2:
                            List<WellDrilling> upperWarnData = hookLoadCatch.getUpperWarnData();
                            upperWarnData.add(nowPoint);
                            hookLoadCatch.setUpperWarnData(upperWarnData);
                            hookLoadCatch.setUpperDrillingToolSteps(3);
                            break;
                        case 3:
                            List<WellDrilling> upperWarnData1 = hookLoadCatch.getUpperWarnData();
                            upperWarnData1.add(nowPoint);
                            hookLoadCatch.setUpperWarnData(upperWarnData1);
                            break;
                        default:
                            break;
                    }
                } else {
                    //下放钻具
                    switch (downDrillingToolSteps) {
                        case 0:
                            List<WellDrilling> downReferenceData = hookLoadCatch.getDownReferenceData();
                            downReferenceData.add(nowPoint);
                            hookLoadCatch.setDownReferenceData(downReferenceData);
                            hookLoadCatch.setDownDrillingToolSteps(1);
                            //当出现下放钻具动作时判断上提钻具钩载
                            hookLoadCatch = upperDrillingToolsHandle(hookLoadCatch, warnValue, nowPoint.getJh(), nowPoint.getMotion(), nowPoint.getOperatingMode());
                            break;
                        case 1:
                            List<WellDrilling> downReferenceData1 = hookLoadCatch.getDownReferenceData();
                            downReferenceData1.add(nowPoint);
                            hookLoadCatch.setDownReferenceData(downReferenceData1);
                            break;
                        case 2:
                            List<WellDrilling> downWarnData = hookLoadCatch.getDownWarnData();
                            downWarnData.add(nowPoint);
                            hookLoadCatch.setDownWarnData(downWarnData);
                            hookLoadCatch.setDownDrillingToolSteps(3);
                            break;
                        case 3:
                            List<WellDrilling> downWarnData1 = hookLoadCatch.getDownWarnData();
                            downWarnData1.add(nowPoint);
                            hookLoadCatch.setDownWarnData(downWarnData1);
                            break;
                        default:
                            break;
                    }
                }
                redisUtil.set(redisHasKey, hookLoadCatch);
            } else {
                DrillingHookLoadCatch catchData = initCatchData(nowPoint, jh, motion, isUpAction, isDownAction);
                redisUtil.set(redisHasKey, catchData);
            }

        } else {
            if (isHave) {
                if (motion.equals(DrillingConditionDeterminationEnum.InSlips.getValue())) {
                    //坐卡则清除上一柱所有缓存数据
                    redisUtil.del(redisHasKey);
                } else if (motion.equals(DrillingConditionDeterminationEnum.RotaryDrilling.getValue())) {
                    //旋转转进判断缓存数据记录步骤
                    DrillingHookLoadCatch hookLoadCatch = JSONObject.toJavaObject(JSONObject.parseObject(JSON.toJSONString(redisUtil.get(redisHasKey))), DrillingHookLoadCatch.class);
                    List<WellDrilling> upperReferenceData = hookLoadCatch.getUpperReferenceData();
                    List<WellDrilling> downReferenceData = hookLoadCatch.getDownReferenceData();
                    hookLoadCatch = upperDrillingToolsHandle(hookLoadCatch, warnValue, nowPoint.getJh(), nowPoint.getMotion(), nowPoint.getOperatingMode());
                    hookLoadCatch = downDrillingToolsHandle(hookLoadCatch, warnValue, nowPoint.getJh(), nowPoint.getMotion(), nowPoint.getOperatingMode());
                    redisUtil.set(redisHasKey, hookLoadCatch);
                }
            }
        }
    }

    /**
     * 上提钻具钩载预警数据判断
     *
     * @param hookLoadCatch
     * @return
     */
    public DrillingHookLoadCatch upperDrillingToolsHandle(DrillingHookLoadCatch hookLoadCatch, BigDecimal warnValue, String jh, String motion, String workingCondition) {
        Integer upperDrillingToolSteps = hookLoadCatch.getUpperDrillingToolSteps();
        if (upperDrillingToolSteps == 1) {
            //下放钻具步骤为记录预警数据中
            List<WellDrilling> upperReferenceData = hookLoadCatch.getUpperReferenceData();
            //计算钩载拨动值
            List<BigDecimal> upperWohList = upperReferenceData.stream().map(WellDrilling::getWoh).collect(Collectors.toList());
            BigDecimal upperReferenceValue = CommonUtils.calculateDifference(upperWohList);
            hookLoadCatch.setUpperReferenceValue(upperReferenceValue);
            hookLoadCatch.setUpperDrillingToolSteps(2);
        } else if (upperDrillingToolSteps == 3) {
            List<WellDrilling> upperReferenceData = hookLoadCatch.getUpperReferenceData();
            //计算钩载拨动值
            List<BigDecimal> upperWohList = upperReferenceData.stream().map(WellDrilling::getWoh).collect(Collectors.toList());
            BigDecimal upperReferenceValue = hookLoadCatch.getUpperReferenceValue();
            List<WellDrilling> upperWarnData = hookLoadCatch.getUpperWarnData();
            List<BigDecimal> upperWarnValueList = upperWarnData.stream().map(WellDrilling::getWoh).collect(Collectors.toList());
            BigDecimal upperWarnValue = CommonUtils.calculateDifference(upperWarnValueList);

            Boolean isWarn = mobileDrillingToolsCalculationRules(upperWarnValue, upperReferenceValue, warnValue);
            if (isWarn) {
                BigDecimal mobilDrillingToolsMaxWoh = upperWarnValueList.stream().min(BigDecimal::compareTo).get();
                BigDecimal mobilDrillingToolsMinWoh = upperWarnValueList.stream().max(BigDecimal::compareTo).get();

                BigDecimal minReferenceWoh = upperWohList.stream().min(BigDecimal::compareTo).get();
                BigDecimal maxReferenceWoh = upperWohList.stream().max(BigDecimal::compareTo).get();
                hookLoadWarnLogService.saveMobilDrillingToolsHookLoadWarnLog(jh, workingCondition, workingCondition + "-活动钻具", motion,
                        upperWarnData.get(0).getTime(), upperWarnData.get(upperWarnData.size() - 1).getTime(), upperWarnValue, mobilDrillingToolsMaxWoh, mobilDrillingToolsMinWoh,
                        upperReferenceData.get(0).getTime(), upperReferenceData.get(upperReferenceData.size() - 1).getTime(), upperReferenceValue,
                        maxReferenceWoh, minReferenceWoh, warnValue);
            }
            log.info("活动钻具预警:参考柱数据{},参考柱波动值:{},当前活动钻具数据:{},当前活动钻具波动值:{}", hookLoadCatch.getDownReferenceData(), upperReferenceValue, upperWarnData, upperWarnValue);
            hookLoadCatch.setUpperWarnData(new ArrayList<>());
            hookLoadCatch.setUpperDrillingToolSteps(2);

        }
        return hookLoadCatch;
    }

    /**
     * 下放钻具钩载预警数据判断
     *
     * @param hookLoadCatch
     * @return
     */
    public DrillingHookLoadCatch downDrillingToolsHandle(DrillingHookLoadCatch hookLoadCatch, BigDecimal warnValue, String jh, String motion, String workingCondition) {
        Integer downDrillingToolSteps = hookLoadCatch.getDownDrillingToolSteps();
        if (downDrillingToolSteps == 1) {
            //下放钻具步骤为记录预警数据中
            List<WellDrilling> downReferenceData = hookLoadCatch.getDownReferenceData();
            List<BigDecimal> downWohList = downReferenceData.stream().map(WellDrilling::getWoh).collect(Collectors.toList());
            BigDecimal downReferenceValue = CommonUtils.calculateDifference(downWohList);
            hookLoadCatch.setDownReferenceValue(downReferenceValue);
            hookLoadCatch.setDownDrillingToolSteps(2);
        } else if (downDrillingToolSteps == 3) {
            List<WellDrilling> downReferenceData = hookLoadCatch.getDownReferenceData();
            List<BigDecimal> downWohList = downReferenceData.stream().map(WellDrilling::getWoh).collect(Collectors.toList());

            BigDecimal downReferenceValue = hookLoadCatch.getDownReferenceValue();
            List<WellDrilling> downWarnData = hookLoadCatch.getDownWarnData();
            List<BigDecimal> downWarnValueList = downWarnData.stream().map(WellDrilling::getWoh).collect(Collectors.toList());
            BigDecimal downWarnValue = CommonUtils.calculateDifference(downWarnValueList);
            Boolean isWarn = mobileDrillingToolsCalculationRules(downWarnValue, downReferenceValue, warnValue);
            if (isWarn) {
                BigDecimal mobilDrillingToolsMaxWoh = downWarnValueList.stream().min(BigDecimal::compareTo).get();
                BigDecimal mobilDrillingToolsMinWoh = downWarnValueList.stream().max(BigDecimal::compareTo).get();

                BigDecimal minReferenceWoh = downWohList.stream().min(BigDecimal::compareTo).get();
                BigDecimal maxReferenceWoh = downWohList.stream().max(BigDecimal::compareTo).get();
                hookLoadWarnLogService.saveMobilDrillingToolsHookLoadWarnLog(jh, workingCondition, workingCondition + "-活动钻具", motion,
                        downReferenceData.get(0).getTime(), downReferenceData.get(downReferenceData.size() - 1).getTime(), downWarnValue, mobilDrillingToolsMaxWoh, mobilDrillingToolsMinWoh,
                        downReferenceData.get(0).getTime(), downReferenceData.get(downReferenceData.size() - 1).getTime(), downReferenceValue,
                        maxReferenceWoh, minReferenceWoh, warnValue);
            }
            log.info("活动钻具预警:参考柱数据{},参考柱波动值:{},当前活动钻具数据:{},当前活动钻具波动值:{}", hookLoadCatch.getDownReferenceData(), downReferenceValue, downWarnData, downWarnValue);
            hookLoadCatch.setDownWarnData(new ArrayList<>());
            hookLoadCatch.setDownDrillingToolSteps(2);
        }
        return hookLoadCatch;
    }

    /**
     * 活动钻具判断规则:当前柱的钩载波动量-这一柱第一次钩载波动量的差值>=50KN
     *
     * @param nowPointWohFluctuatedValue   当前柱的钩载波动量
     * @param firstPointWohFluctuatedValue 这一柱第一次钩载波动量
     * @param warnValue                    预警值
     * @return true:报警/false:不报警
     */
    public Boolean mobileDrillingToolsCalculationRules(BigDecimal nowPointWohFluctuatedValue, BigDecimal firstPointWohFluctuatedValue, BigDecimal warnValue) {
        int res = -1;
        BigDecimal fluctuatedValueSubtract = nowPointWohFluctuatedValue.subtract(firstPointWohFluctuatedValue);
        res = fluctuatedValueSubtract.compareTo(warnValue);
        return res >= 0;
    }

    /**
     * 初始化缓存数据
     *
     * @param nowPoint 当前点数据
     * @param jh       井号
     * @param motion   当前点动作
     * @return 缓存数据
     */
    private DrillingHookLoadCatch initCatchData(WellDrilling nowPoint, String jh, String motion, Boolean isUpAction, Boolean isDownAction) {
        DrillingHookLoadCatch catchData = new DrillingHookLoadCatch();
        catchData.setJh(jh);
        List<WellDrilling> columnList = new ArrayList<>();
        columnList.add(nowPoint);
        catchData.setDownReferenceValue(BigDecimal.valueOf(0));
        catchData.setUpperReferenceValue(BigDecimal.valueOf(0));
        catchData.setUpperWarnData(new ArrayList<>());
        catchData.setDownWarnData(new ArrayList<>());
        if (isUpAction) {
            //上提钻具动作
            catchData.setUpperReferenceData(columnList);
            catchData.setUpperDrillingToolSteps(1);
            catchData.setDownReferenceData(new ArrayList<>());
            catchData.setDownDrillingToolSteps(0);
        } else if (isDownAction) {
            //下放钻具动作
            catchData.setDownReferenceData(columnList);
            catchData.setDownDrillingToolSteps(1);
            catchData.setUpperReferenceData(new ArrayList<>());
            catchData.setUpperDrillingToolSteps(0);
        } else {
            catchData.setDownReferenceData(new ArrayList<>());
            catchData.setDownDrillingToolSteps(0);
            catchData.setUpperReferenceData(new ArrayList<>());
            catchData.setUpperDrillingToolSteps(0);
        }
        return catchData;
    }

}
