package com.cn.tianji.service.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cn.tianji.common.CommonUtils;
import com.cn.tianji.common.PumpingPressureCatch;
import com.cn.tianji.common.RedisUtil;
import com.cn.tianji.entity.PumpingPressureWarnLog;
import com.cn.tianji.entity.WarnHistory;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.enums.DrillingActionPumpingEnum;
import com.cn.tianji.enums.TrippingActionPumpingEnum;
import com.cn.tianji.service.PumpingPressureService;
import com.cn.tianji.service.PumpingPressureWarnLogService;
import com.cn.tianji.service.WarnHistoryService;
import com.cn.tianji.service.WellDrillingService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 泵压预警计算服务层
 *
 * @author Zhao RenJi
 */
@Slf4j
@Service
public class PumpingPressureServiceImpl implements PumpingPressureService {

    @Resource
    private WellDrillingService wellDrillingService;

    @Resource
    private WarnHistoryService warnHistoryService;

    @Resource
    private PumpingPressureWarnLogService pumpingPressureWarnLogService;

    @Resource
    private RedisUtil redisUtil;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    /**
     * 参与判断因素:总泵冲、泵压
     * 在钻进工况中先判断:
     * 当前点总泵冲-1分钟前总泵冲/1分钟前总泵冲<5% （先判断总泵冲）
     * 继续判断当前点泵压-1分钟前点泵压/1分钟前泵压>10% 报警
     * （停泵后重新启动后，当总泵冲发生较大变化时,泵压再出现预警时，判断预警点与泵冲发生变化的开始点位的时间间隔是否超过5分钟
     * 当泵冲发生变化后，泵冲再次稳定之后，3分钟之内出现的泵压预警不计入预警数据）
     */
    @SneakyThrows
    @Override
    public void pumpingPressureCalculate(WellDrilling nowPoint, String redisHasKey, BigDecimal pumpingWarnValue,BigDecimal sppWarnValue) {
        PumpingPressureCatch pumpingPressureCatch = new PumpingPressureCatch();
        String jh = nowPoint.getJh();
        Date nowDate=new Date();
        BigDecimal warnValue = BigDecimal.valueOf(0.05);
        BigDecimal warnValueSpp = BigDecimal.valueOf(0.1);
//        String redisHasKey = jh + "_pumpingPressure";
        boolean isHaveCatch = redisUtil.hasKey(redisHasKey);
        String motion = nowPoint.getMotion();
        boolean isWarnTrippingAction = CommonUtils.diagnosisWarnAction(TrippingActionPumpingEnum.class, motion);
        boolean isWarnDrillingAction = CommonUtils.diagnosisWarnAction(DrillingActionPumpingEnum.class, motion);
        if (isWarnTrippingAction || isWarnDrillingAction) {
            log.info("判断泵压开始");
            if (isHaveCatch) {
                pumpingPressureCatch = JSONObject.toJavaObject(JSONObject.parseObject(JSON.toJSONString(redisUtil.get(redisHasKey))), PumpingPressureCatch.class);
                Integer calStep = pumpingPressureCatch.getCalStep();
                BigDecimal totalPumpBig = calculateTotalPump(nowPoint);
                boolean isZeroNow = totalPumpBig.compareTo(BigDecimal.ZERO) > 0;
                //判断当前点泵冲是否为0 如果为零则重新记录数据且记录该点为泵冲变化起点
                if(!isZeroNow){
                    List<WellDrilling> perList = new ArrayList<>();
                    pumpingPressureCatch.setCalStep(0);
                    pumpingPressureCatch.setPumpChangePoint(nowPoint);
                    pumpingPressureCatch.setPerPoint(perList);
                }else{
                    switch (calStep){
                        case  0:
                            log.info("判断泵压记录泵压数据");
                            List<WellDrilling> perList = new ArrayList<>();
                            perList.add(nowPoint);
                            pumpingPressureCatch.setCalStep(1);
                            pumpingPressureCatch.setPumpChangePoint(nowPoint);
                            pumpingPressureCatch.setPerPoint(perList);
                            break;
                        case 1:
                            log.info("判断泵压记录泵压数据1");
                            List<WellDrilling> perPoints = pumpingPressureCatch.getPerPoint();
                            WellDrilling perPoint = perPoints.get(0);
                            long timeDiff = CommonUtils.timeDifference(sdf.parse(perPoint.getTime()), sdf.parse(nowPoint.getTime()));
                            long timeDiffValue = 60;
                            if (timeDiff >= timeDiffValue) {
                                pumpingPressureCatch=warnCalculate( perPoint, nowPoint, warnValue, warnValueSpp, pumpingPressureCatch);
                                //pumpingPressureCatch.setCalStep(2);
                            }else{
                                perPoints.add(nowPoint);
                                pumpingPressureCatch.setPerPoint(perPoints);
                            }
                            break;
                        case 2:
                            log.info("开始计算预警数据");
                            List<WellDrilling> perPointss = pumpingPressureCatch.getPerPoint();
                            WellDrilling perPointc = perPointss.get(0);
                            pumpingPressureCatch=warnCalculate( perPointc, nowPoint, warnValue, warnValueSpp, pumpingPressureCatch);
                            break;
                        default:
                            break;
                    }
                }
            } else {
                //如果没有缓存数据则开始记录缓存数据
                List<WellDrilling> perList = new ArrayList<>();
                perList.add(nowPoint);
                pumpingPressureCatch.setCalStep(1);
                pumpingPressureCatch.setJh(jh);
                pumpingPressureCatch.setPumpChangePoint(nowPoint);
                pumpingPressureCatch.setPerPoint(perList);
                //redisUtil.set(redisHasKey,pumpingPressureCatch);
            }
            redisUtil.set(redisHasKey, pumpingPressureCatch);
        }else{
            redisUtil.del(redisHasKey);
        }
        Date endDate=new Date();
        long warnTimeDiff = CommonUtils.timeDifference_(nowDate, endDate);
        log.info("计算耗时:{}毫秒",warnTimeDiff);
    }

    @SneakyThrows
    public PumpingPressureCatch warnCalculate(WellDrilling perPoint, WellDrilling nowPoint,
                                              BigDecimal pumpingWarnValue,BigDecimal sppWarnValue,
                                              PumpingPressureCatch pumpingPressureCatch){
        BigDecimal perTotalPumpBig = calculateTotalPump(perPoint);
        BigDecimal totalPumpBig = calculateTotalPump(nowPoint);
        boolean isNoZero = totalPumpingIsZero(perPoint, nowPoint);
        if (isNoZero) {
            Boolean isPumpWarn = pumpCalculate(perTotalPumpBig, totalPumpBig, pumpingWarnValue);
            //总泵冲是否发生变化
            if (!isPumpWarn) {
                WellDrilling pumpChangePoint = pumpingPressureCatch.getPumpChangePoint();
                //泵冲变化起点时间
                if(pumpChangePoint!=null){
                    //泵冲变化起点时间
                    long timeDiff = CommonUtils.timeDifference(sdf.parse(pumpChangePoint.getTime()), sdf.parse(nowPoint.getTime()));
                    long timeDiffValue = 3*60;
                    if(timeDiff>=timeDiffValue){
                        pumpingPressureCatch=  warnCal(perPoint,  nowPoint, pumpingWarnValue, sppWarnValue,pumpingPressureCatch, totalPumpBig, perTotalPumpBig);
                    }else{
                        List<WellDrilling> perPointList = pumpingPressureCatch.getPerPoint();
                        perPointList.remove(0);
                        perPointList.add(nowPoint);
                        pumpingPressureCatch.setPerPoint(perPointList);
                    }
                }else{
                    pumpingPressureCatch= warnCal(perPoint,  nowPoint,  pumpingWarnValue, sppWarnValue,  pumpingPressureCatch, totalPumpBig, perTotalPumpBig);
                }
                Integer calStep = pumpingPressureCatch.getCalStep();
                if(calStep==1){
                    pumpingPressureCatch.setCalStep(2);
                }
            }else{
                log.info("波动点时间:{}",nowPoint.getTime());
                List<WellDrilling> perList = new ArrayList<>();
//                perList.add(nowPoint);
                pumpingPressureCatch.setCalStep(0);
                pumpingPressureCatch.setPumpChangePoint(nowPoint);
                pumpingPressureCatch.setPerPoint(perList);
            }
        }else{
            List<WellDrilling> perList = new ArrayList<>();
//                perList.add(nowPoint);
            pumpingPressureCatch.setCalStep(0);
            pumpingPressureCatch.setPumpChangePoint(nowPoint);
            pumpingPressureCatch.setPerPoint(perList);
        }
        return pumpingPressureCatch;
    }

    /**
     * 判断当前点与参考点总泵冲是否为0
     * @param perPoint
     * @param nowPoint
     * @return 大于零 true 小于等于零 false
     */
    public boolean totalPumpingIsZero(WellDrilling perPoint, WellDrilling nowPoint){
        BigDecimal perTotalPumpBig = calculateTotalPump(perPoint);
        BigDecimal totalPumpBig = calculateTotalPump(nowPoint);
        boolean isZeroNow = totalPumpBig.compareTo(BigDecimal.ZERO) > 0;
        boolean isZeroPer = perTotalPumpBig.compareTo(BigDecimal.ZERO) > 0;
       return isZeroNow && isZeroPer;
    }

    public PumpingPressureCatch warnCal(WellDrilling perPoint, WellDrilling nowPoint,
                                        BigDecimal pumpingWarnValue,BigDecimal sppWarnValue,
                                        PumpingPressureCatch pumpingPressureCatch,BigDecimal totalPumpBig,BigDecimal perTotalPumpBig){
        //未发生变化
        BigDecimal perSpp = perPoint.getSpp().setScale(2, RoundingMode.HALF_UP);
        BigDecimal nowSpp = nowPoint.getSpp().setScale(2, RoundingMode.HALF_UP);
        boolean isZeroSppNow = nowSpp.compareTo(BigDecimal.ZERO) > 0;
        boolean isZeroSppPer = perSpp.compareTo(BigDecimal.ZERO) > 0;
        if (isZeroSppNow && isZeroSppPer) {
            log.info("当前点泵压时间:{};参考点泵压时间:{}", nowPoint.getTime(), CommonUtils.removeMilliseconds(perPoint.getTime()));
//            int isBoundaryValue = nowSpp.compareTo(BigDecimal.valueOf(30));
//            if(isBoundaryValue>=0){
//                warnValueSpp=BigDecimal.valueOf(0.05);
//            }else{
//                warnValueSpp=BigDecimal.valueOf(0.1);
//            }
            Boolean isWarn = sppCalculate(perSpp, nowSpp, sppWarnValue);
            log.info("是否预警:{}", isWarn ? "是" : "否");
            if (isWarn) {
                BigDecimal pumpSubtract = totalPumpBig.subtract(perTotalPumpBig).setScale(2,RoundingMode.HALF_UP);
                BigDecimal sppSubtract = nowSpp.subtract(perSpp).setScale(2,RoundingMode.HALF_UP);
                pumpingPressureCatch=savePumpingPressureWarnLog(nowPoint,totalPumpBig,perPoint,perTotalPumpBig,pumpingPressureCatch,pumpSubtract,sppSubtract,sppWarnValue.multiply(BigDecimal.valueOf(100)),pumpingWarnValue.multiply(BigDecimal.valueOf(100)));
            }else {
                List<WellDrilling> perPoints = pumpingPressureCatch.getPerPoint();
                perPoints.remove(0);
                perPoints.add(nowPoint);
                pumpingPressureCatch.setPerPoint(perPoints);
            }
        }
        return pumpingPressureCatch;
    }

    /**
     * 点泵冲判断:当前点总泵冲-1分钟前总泵冲/1分钟前总泵冲<5%
     *
     * @param perTotalPump 上一点总泵冲
     * @param nowTotalPump 当前点总泵冲
     * @param warnValue    预警值
     */
    public Boolean pumpCalculate(BigDecimal perTotalPump, BigDecimal nowTotalPump, BigDecimal warnValue) {
        BigDecimal pumpSubtract = nowTotalPump.subtract(perTotalPump).setScale(2,RoundingMode.HALF_UP).abs();
        BigDecimal pumpDivide = pumpSubtract.divide(perTotalPump, 2, RoundingMode.HALF_UP);
        log.info("泵冲计算公式:当前点泵冲{}-1分钟前总泵冲{}={}",nowTotalPump,perTotalPump,pumpSubtract);
        log.info("泵冲计算公式:泵冲差值{}÷1分钟前总泵冲{}={}",pumpSubtract,perTotalPump,pumpDivide);
        return pumpDivide.compareTo(warnValue) > 0;
    }

    /**
     * 点立压判断:当前点泵压-1分钟前点泵压/1分钟前泵压>10%
     *
     * @param perSpp    上一点总泵冲
     * @param nowSpp    当前点总泵冲
     * @param warnValue 预警值
     */
    public Boolean sppCalculate(BigDecimal perSpp, BigDecimal nowSpp, BigDecimal warnValue) {
        BigDecimal sppSubtract = nowSpp.subtract(perSpp).setScale(2,RoundingMode.HALF_UP).abs();
        BigDecimal sppDivide = sppSubtract.divide(perSpp, 2, RoundingMode.HALF_UP);
        log.info("泵压计算公式:当前点泵压:{}-1分钟前泵压:{}={}", nowSpp, perSpp, sppSubtract);
        log.info("泵压计算公式:泵压差值:{}/1分钟前泵压:{}={}", sppSubtract, perSpp, sppDivide);
        return sppDivide.compareTo(warnValue) > 0;
    }

    /**
     * 计算总泵冲
     *
     * @param point 点位数据
     * @return 总泵冲
     */
    public BigDecimal calculateTotalPump(WellDrilling point) {
        BigDecimal pumpB1 = point.getPump1();
        BigDecimal pumpB2 = point.getPump2();
        BigDecimal pumpB3 = point.getPump3();
        BigDecimal zero = BigDecimal.valueOf(0);
        BigDecimal res = BigDecimal.valueOf(0);
        //判断三个泵冲是否有参数值;负值为无参数
        if (pumpB1.compareTo(zero) >= 0 && pumpB2.compareTo(zero) >= 0 && pumpB3.compareTo(zero) >= 0) {
            res = pumpB1.add(pumpB2).add(pumpB3);
        } else {
            if (pumpB1.compareTo(zero) >= 0 && pumpB2.compareTo(zero) >= 0) {
                res = pumpB1.add(pumpB2);
            } else if (pumpB1.compareTo(zero) >= 0 && pumpB3.compareTo(zero) >= 0) {
                res = pumpB1.add(pumpB3);
            } else if (pumpB2.compareTo(zero) >= 0 && pumpB3.compareTo(zero) >= 0) {
                res = pumpB2.add(pumpB2);
            } else {
                if (pumpB1.compareTo(zero) >= 0) {
                    res = pumpB1;
                } else if (pumpB2.compareTo(zero) >= 0) {
                    res = pumpB2;
                } else if (pumpB3.compareTo(zero) >= 0) {
                    res = pumpB3;
                }
            }
        }
        return res;
    }

    @SneakyThrows
    public PumpingPressureCatch savePumpingPressureWarnLog(WellDrilling nowPoint,BigDecimal nowPointTotalPump,
                                           WellDrilling referencePoint,BigDecimal referenceTotalPump,
                                           PumpingPressureCatch pumpingPressureCatch,BigDecimal totalPumpSub,BigDecimal sppSub,BigDecimal sppWarnValue,BigDecimal pumpWarnValue ){
        String jh=nowPoint.getJh();
        Date warnTime=sdf.parse(nowPoint.getTime());
        Date lastWarnDate = pumpingPressureCatch.getLastWarnDate();
        PumpingPressureWarnLog pumpingPressureWarnLog = pumpingPressureWarnLogService.createPumpingPressureWarnLog(jh, nowPoint.getOperatingMode(), nowPoint.getMotion(), warnTime, nowPoint.getSpp(), nowPointTotalPump,
                sdf.parse(referencePoint.getTime()), referencePoint.getSpp(), referenceTotalPump, totalPumpSub, sppSub, sppWarnValue, pumpWarnValue);
        if(lastWarnDate!=null){
            long warnTimeDiff = CommonUtils.timeDifference(lastWarnDate, warnTime);
            if(warnTimeDiff >= 15){
                //大于10秒说明不是连续的预警数据，重新记录
                List<PumpingPressureWarnLog> warnList=new ArrayList<>();
                //起点
                pumpingPressureWarnLog.setDataType("0");
                warnList.add(pumpingPressureWarnLog);
                pumpingPressureCatch.setWarnList(warnList);
            }else{
                List<PumpingPressureWarnLog> warnList = pumpingPressureCatch.getWarnList();

                warnList.add(pumpingPressureWarnLog);
                //结束点
                pumpingPressureWarnLog.setDataType("1");
                if(warnList.size()>=3){
                    pumpingPressureWarnLogService.save(pumpingPressureWarnLog);
//                    QueryWrapper<PumpingPressureWarnLog> wrapper = new QueryWrapper<>();
//                    wrapper.eq("JH",jh);
//                    wrapper.apply("WARN_TIME= to_date({0},'yyyy-mm-dd hh24:mi:ss')",sdf.format(lastWarnDate));
//                    List<PumpingPressureWarnLog> list = pumpingPressureWarnLogService.list(wrapper);
//                    if(list.size()>0){
//                        PumpingPressureWarnLog updateModel = list.get(0);
//                        pumpingPressureWarnLog.setId(updateModel.getId());
//                        pumpingPressureWarnLogService.updateById(pumpingPressureWarnLog);
//                    }else{
//                        PumpingPressureWarnLog pumpingPressureWarnLogStart = warnList.get(0);
//                        pumpingPressureWarnLogService.save(pumpingPressureWarnLogStart);
//
//                    }
                }
                pumpingPressureCatch.setWarnList(warnList);
            }
        }else{
            List<PumpingPressureWarnLog> warnList=new ArrayList<>();
            warnList.add(pumpingPressureWarnLog);
            pumpingPressureCatch.setWarnList(warnList);
        }
        pumpingPressureCatch.setLastWarnDate(warnTime);
        return pumpingPressureCatch;
    }

    /**
     * 插入预警数据
     *
     * @param nowPoint
     * @param wholeCondition
     * @param perTime
     * @param NormalWoh
     * @param nowSpp
     * @param perSpp
     * @param warnValue
     * @param loopDrillingSpp
     */
    @SneakyThrows
    public void saveWarnHistory(WellDrilling nowPoint, String wholeCondition, Date perTime, Float NormalWoh, Float nowSpp, Float perSpp,
                                Float warnValue, Float loopDrillingSpp) {
        String des = "符合泵压变化异常预警条件(当前点立压:" + nowPoint.getSpp() + " Mpa," + "当前点时间:" + CommonUtils.removeMilliseconds(nowPoint.getTime()) +
                "参考点立压:" + perSpp + " Mpa,参考点时间:" + sdf.format(perTime) +
                "大于立压报警阈值" + warnValue + "%);";
        WarnHistory history = new WarnHistory();
        history.setJh(nowPoint.getJh());
        history.setWarnType("泵压变化异常");
        history.setWarnTime(sdf.parse(CommonUtils.removeMilliseconds(nowPoint.getTime())));
        history.setWarnHkh(nowPoint.getHkh().floatValue());
        history.setWarnBitdepth(nowPoint.getBitdepth().floatValue());
        history.setWarnWoh(nowSpp);
        history.setWarnCondition(wholeCondition);
        history.setWholeCondition(wholeCondition);
        history.setNormalStartime(perTime);
        history.setNormalWoh(NormalWoh);
        history.setPerSpp(perSpp);
        history.setLoopDrillingPump(warnValue);
        history.setLoopDrillingSpp(loopDrillingSpp);
        history.setWarnDes(des);
        warnHistoryService.save(history);
    }
}
